import uuid
from PySide6.QtWidgets import QVBoxLayout, QFrame
from typing import Union, List, Callable

class TimelineSeparator(QFrame):
    def __init__(self, 
                 children=None,  # Nội dung của component (node)
                 classes: dict = None,  # Ghi đè hoặc mở rộng các styles áp dụng cho component (dict)
                 sx: Union[List[Union[Callable, dict, bool]], Callable, dict] = None  # Thuộc tính hệ thống, thêm CSS bổ sung (array | func | object)
                 ):
        super().__init__()
        self.setObjectName(str(uuid.uuid4()))
        # self.setStyleSheet(f"#{self.objectName()} {{background-color: green;}}")

        self._children = children
        self._classes = classes if classes is not None else {}
        self._sx = sx if sx is not None else []

        self._initUI()

    def _initUI(self):
        """Khởi tạo giao diện của TimelineItem."""
        self.setLayout(QVBoxLayout())
        self.layout().setContentsMargins(2,2,2,2)
        
        self._add_children()

    # @is_list
    def _add_children(self):
        # Thêm các children (nếu có)
        if self._children:
            if not isinstance(self._children, list):
                raise TypeError("children must be type (list)")
            for child in self._children:
                if child:
                    self.layout().addWidget(child)



import uuid
from typing import Optional, Callable
from PySide6.QtWidgets import QVBoxLayout, QFileDialog, QFrame, QPushButton
from PySide6.QtCore import Qt, QSize
from PySide6.QtGui import QCursor, QMouseEvent

from ..typography import Typography
from ..box import Box
from ..py_iconify import PyIconify, Iconify
from ..py_svg_widget import PySvgWidget
from ...qtmui_assets import QTMUI_ASSETS
from ..button.button import Button
from qtmui.material.styles import useTheme



styles = """
        #{} {{
            border: {};
            margin: {};
            font-weight: {};
            line-height: {};
            font-size: {};
            font-family: {};
            padding: {};
            border-radius: {};
            color:  {};
            background-color: {};
        }}
        #{}::hover {{
            background-color: {};
        }}

"""


class UploadBox(QFrame):
# class Chip(QWidget):
    """
    Button
    Base Chip

    Args:
        key?: 'filled' | 'outlined' | 'outlined' | 'soft'
        severity?: str = 'info' | 'success' | 'warning' | 'error'
        variant?: 'filled' | 'outlined' | 'outlined' | 'soft'
        size?: str = 'small' | 'medium' | 'large'
        sx: str = QSS string
            {
                color: red;
                backroud: none;
            }
        justifyContent: "space-around" | "space-between" | "space-evenly" | "stretch" | "center" | "end" | "flex-end" | "flex-start" | "start"

    Returns:
        new instance of Button
    """
    def __init__(self,
                files: str = None,
                onDrop: Optional[Callable] = None,
                error: object = None,
                placeholder: object = None,
                sx: object = None,
                 *args, 
                 **kwargs
                 ):
        super().__init__()

        self._files = files
        self._onDrop = onDrop
        self._error = error
        self._placeholder = placeholder
        self._sx = sx

        self._init_ui()

    def _init_ui(self):
        self.setObjectName(str(uuid.uuid4()))

        self.setLayout(QVBoxLayout())
        self.layout().setContentsMargins(0,0,0,0)
        self.setCursor(QCursor(Qt.PointingHandCursor))

        self.layout().addWidget(
            Box(
                direction="column",
                children=[
                    self._placeholder if self._placeholder else
                    Box(
                        children=[
                            Box(
                                direction="row",
                                children=[
                                    Button(
                                        variant="soft", 
                                        text="Upload", 
                                        icon=Iconify(key=QTMUI_ASSETS.ICONS.CLOUD_UPLOAD_BOLD, color=useTheme().palette.primary.main, size=QSize(200, 200)), 
                                        onClick=self.open_file
                                    ),
                                    Box(
                                        direction="column",
                                        children=[
                                            Typography(text="Allowed *.jpeg, *.jpg, *.png, *.gif", variant="h6"),
                                            Typography(text="Max size of 3 Mb", variant="h6")
                                        ]
                                    )
                                ]
                            ),
                            Typography(text="", variant="h5")
                        ]
                    )
                ]
            )
        )


        # self.setStyleSheet(styles.format(
        #     self.objectName(),
        #     self._border,
        #     self._margin,
        #     self._fontWeight,
        #     self._lineHeight,
        #     self._fontSize,
        #     self._fontFamily,
        #     self._padding,
        #     self._borderRadius,
        #     self._textColor, 
        #     self._backgroundColor,
        #     self.objectName(),
        #     self._hoverBackgroundColor
        # ))

    def mousePressEvent(self, event: QMouseEvent) -> None:
        self.open_file()
        return super().mousePressEvent(event)

    def open_file(self):
        file_name, _ = QFileDialog.getOpenFileName(self, "Open File", "", "Text Files (*.txt);;All Files (*)")
        self.lbl_file_info.setText(file_name)

    def format_size(self, bytes):
        """
        Hàm này nhận đầu vào là số byte và trả về chuỗi định dạng kích thước tương ứng
        dưới dạng byte, KB, MB hoặc GB.
        """
        if bytes < 1024:
            return f"{bytes} B"
        elif bytes < 1024**2:
            kb = bytes / 1024
            return f"{kb:.2f} KB"
        elif bytes < 1024**3:
            mb = bytes / 1024**2
            return f"{mb:.2f} MB"
        else:
            gb = bytes / 1024**3
            return f"{gb:.2f} GB"


import asyncio
from PySide6.QtWidgets import QHBoxLayout, QVBoxLayout, QFrame, QWidget, QScrollArea, QSizePolicy, QApplication, QLabel
from PySide6.QtGui import QGuiApplication
from PySide6.QtCore import Qt, QRunnable, QTimer, QThreadPool
import uuid

from qtmui.hooks import State

class WidgetSetter(QRunnable):
    def __init__(self, cls, children):
        super().__init__()
        self.cls= cls
        self.children=children

    async def run(self):
        for index, widget in enumerate(self.children):
            await asyncio.sleep(1)
            print(f"add widget {index} before: ", self.cls.height())
            self.cls.vlayout.addWidget(widget)
            self.cls.widgetContents.adjustSize()
            self.cls.scroll_area.widget().adjustSize()
            self.cls.scroll_area.updateGeometry()
            self.cls.updateGeometry()
            print(f"add widget {index} after: ", self.cls.height())

    
class MenuScrollArea(QFrame):

    def __init__(self,  
                sx: dict = None,
                maxWidth: int = None,
                defaultWidth: int = None,
                hightLight: bool = None,
                backgroundColor: str = None,
                children: list = None,
                fullWidth: bool = False,
                spacing=6,
                border: str = '',
                borderForNav: bool = None,
        ):
        super().__init__()
        self.setObjectName(str(uuid.uuid4()))
        self.setLayout(QVBoxLayout())
        self.layout().setContentsMargins(0,0,0,0)
        self.layout().setSpacing(spacing)
        self._border = border
        
        self._thread_pool = QThreadPool.globalInstance()  # Sử dụng thread pool toàn cục
        
        self.scroll_area = QScrollArea(self)
        self.layout().addWidget(self.scroll_area)

        self.scroll_area.setWidgetResizable(True)
        self.widgetContents = QWidget(self.scroll_area)

        self.vlayout = QVBoxLayout(self.widgetContents)
        self.vlayout.setContentsMargins(0,0,0,0)
        self.vlayout.setSpacing(3)
        
        self.scroll_area.setWidget(self.widgetContents)
            

        if backgroundColor is not None:
            if isinstance(backgroundColor, State):
                backgroundColor.valueChanged.connect(self._set_background)
                self._set_background(backgroundColor.value)
            else:
                self._set_background(background=backgroundColor)

        if maxWidth is not None:
            self.setMaximumWidth(maxWidth)

        if defaultWidth is not None:
            self.resize(defaultWidth, self.height())

        # if sx is not None:
        #     self.setStyleSheet(f'''#{self.objectName()}  {{ {sx} }}''')

        # print('children_____________________', children)

        # if isinstance(children, list) and len(children) > 0:
        #     for item in children:
        #         if isinstance(item, QWidget):
        #             # self.vlayout.addWidget(item)
        #             self._add_child_fast(item)
        #             # print('itemmmmmmmmmmmmm', item)

        self.worker = WidgetSetter(self, children)
        self._thread_pool.start(QTimer.singleShot(0, lambda: asyncio.ensure_future(self.worker.run())))


        if fullWidth:
            self.setFullWidth()

        # self.installEventFilter(self)
        self._set_stylesheet()
        self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Maximum)

    def _add_child_fast(self, widget):
        self._thread_pool.start(QTimer.singleShot(0, lambda widget=widget: asyncio.ensure_future(self.worker.run())))  # Chạy setIndexWidget trong thread riêng

    def _set_stylesheet(self):
        # print('_set_background__________container', background)
        # print('_set_background___________border', self._border)
        self.setStyleSheet(f"""

            QWidget {{
                background-color: transparent;
            }}
            QScrollBar:horizontal {{
                border: none;
                background: rgb(249, 249, 249);
                height: 4px;
                margin: 0px 21px 0 21px;
                border-radius: 0px;
            }}

            QScrollBar::handle:horizontal {{
                background: rgb(227, 227, 227);
                min-width: 25px;
                border-radius: 0px
            }}

            QScrollBar::add-line:horizontal {{
                border: none;
                background: rgb(227, 227, 227);
                width: 20px;
                border-top-right-radius: 0px;
                border-bottom-right-radius: 0px;
                subcontrol-position: right;
                subcontrol-origin: margin;
            }}

            QScrollBar::sub-line:horizontal {{
                border: none;
                background: transparent;
                width: 20px;
                border-top-left-radius: 0px;
                border-bottom-left-radius: 0px;
                subcontrol-position: left;
                subcontrol-origin: margin;
            }}

            QScrollBar::up-arrow:horizontal,
            QScrollBar::down-arrow:horizontal {{
                background: none;
            }}

            QScrollBar::add-page:horizontal,
            QScrollBar::sub-page:horizontal {{
                background: none;
            }}

            QScrollBar:vertical {{
                border: none;
                background: rgb(249, 249, 249);
                width: 4px;
                margin: 21px 0 21px 0;
                border-radius: 0px;
            }}

            QScrollBar::handle:vertical {{
                background: rgb(227, 227, 227);
                min-height: 25px;
                border-radius: 0px;
            }}


            QScrollBar::add-line:vertical {{
                border: none;
                background: transparent;
                height: 20px;
                border-bottom-left-radius: 0px; 
                border-bottom-right-radius: 0px;
                subcontrol-position: bottom;
                subcontrol-origin: margin;
            }}

            QScrollBar::sub-line:vertical {{
                border: none;
                background: transparent;
                height: 20px;
                border-top-left-radius: 0px;
                border-top-right-radius: 0px;
                subcontrol-position: top;
                subcontrol-origin: margin;
            }}

            QScrollBar::up-arrow:vertical,
            QScrollBar::down-arrow:vertical {{
                background: none;
            }}

            QScrollBar::add-page:vertical,
            QScrollBar::sub-page:vertical {{
                background: none;
            }}
        """)

    def _set_background(self, background):
        self.setStyleSheet(self.styleSheet() + f"""
            #{self.objectName()} {{
                background-color: {background};
                {self._border}
            }}
        """)


    def setFullWidth(self):
        self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        self.layout().setContentsMargins(0, 0, 0, 0)
        self.vlayout.setContentsMargins(0, 0, 0, 0)
        self.vlayout.setSpacing(0)
        self.scroll_area.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.scroll_area.setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)

    # def eventFilter(self, obj, event):
    #     if event.type() == QEvent.Resize:
    #         self.adjustWidth()
    #     return super().eventFilter(obj, event)

    def adjustWidth(self):
        screen_size = QGuiApplication.primaryScreen().size()
        screen_width = screen_size.width()
        if screen_width < 576:
            self.setMaximumWidth(screen_width)
        elif screen_width < 768:
            self.setMaximumWidth(540)
        elif screen_width < 992:
            self.setMaximumWidth(720)
        elif screen_width < 1200:
            self.setMaximumWidth(960)
        elif screen_width < 1400:
            self.setMaximumWidth(1140)
        else:
            self.setMaximumWidth(1320)


from .py_combobox import PyComboBox
from .py_date_edit import PyDateEdit
from .py_date_time_edit import PyDateTimeEdit
from .py_dial import PyDial
from .py_double_spinbox import PyDoubleSpinBox
from .py_filled_input import MuiFilledInput
from .py_font_combobox import PyFontComboBox
from .py_input_base import MuiInputBase
from .py_input import MuiInput
from .py_line_edit import PyLineEdit
from .py_outlined_input import MuiOutlinedInput
from .py_plaintext_edit import PyPlainTextEdit
# from .p_search_combobox import PS
from .py_spin_box import PySpinBox
from .py_standard_input import MuiStandardInput
from .py_text_edit import PyTextEdit
from .textfield import TextField



import sys
from PySide6.QtWidgets import (
    QApplication,
    QMainWindow,
    QFrame,
    QLabel,
    QVBoxLayout,
    QWidget,
)

from PySide6.QtCore import (
    Qt,
    QObject,
    Property,
    QPropertyAnimation,
    QEasingCurve,
    QPointF,
)
from PySide6.QtGui import QLinearGradient, QColor, QBrush




class BackgroundPanEffect(QObject):
    """
    BackgroundPanEffect "pro"

    - GIỮ chiến lược mượt:
        Gradient rất dài + dịch geometry
    """

    PRESET_COLORS = {
        "ocean": {
            "light": ("#2193b0", "#6dd5ed"),
            "dark": ("#141e30", "#243b55"),
        },
        "sunset": {
            "light": ("#ff512f", "#f09819"),
            "dark": ("#3a1c71", "#d76d77"),
        },
        "forest": {
            "light": ("#11998e", "#38ef7d"),
            "dark": ("#0f2027", "#203a43"),
        },
    }

    def __init__(
        self,
        widget: QWidget,
        variant="panRight",
        colors="ocean",
        interpolate="rgb",
        steps=4,
        duration=10000,
        theme="light",
    ):
        super().__init__(widget)

        self._widget = widget
        self._variant = variant
        self._interpolate = interpolate
        self._steps = max(2, steps)
        self._duration = duration
        self._theme = theme

        self._colors = self._resolve_colors(colors)

        self._offset = 0.0

        self._anim = QPropertyAnimation(self, b"offset", self)
        self._anim.setStartValue(0.0)
        self._anim.setEndValue(1.0)
        self._anim.setDuration(self._duration)
        self._anim.setLoopCount(-1)
        self._anim.setEasingCurve(QEasingCurve.Linear)
        self._anim.start()

    # ---------------- Property ----------------
    def getOffset(self):
        return self._offset

    def setOffset(self, value):
        self._offset = value
        self._apply_gradient()

    offset = Property(float, getOffset, setOffset)

    # ---------------- Resolve colors ----------------
    def _resolve_colors(self, colors):
        if isinstance(colors, str):
            preset = self.PRESET_COLORS.get(colors)
            return preset[self._theme]
        return tuple(colors)

    # ---------------- Generate colors ----------------
    def _generate_colors(self):
        start = QColor(self._colors[0])
        end = QColor(self._colors[1])

        result = []

        for i in range(self._steps):
            t = i / (self._steps - 1)

            if self._interpolate == "hsl":
                c = QColor.fromHsl(
                    int(start.hslHue() + t * (end.hslHue() - start.hslHue())),
                    int(start.hslSaturation() + t * (end.hslSaturation() - start.hslSaturation())),
                    int(start.lightness() + t * (end.lightness() - start.lightness())),
                )
            else:
                c = QColor(
                    int(start.red() + t * (end.red() - start.red())),
                    int(start.green() + t * (end.green() - start.green())),
                    int(start.blue() + t * (end.blue() - start.blue())),
                )

            result.append(c)

        # DEBUG
        # print("[DEBUG] Colors:", [c.name() for c in result])
        return result

    # =====================================================
    # 🔥 FIX MƯỢT Ở ĐÂY – DỊCH GEOMETRY, KHÔNG DỊCH COLOR
    # =====================================================
    def _apply_gradient(self):
        rect = self._widget.rect()
        colors = self._generate_colors()

        fill_factor = 6  # CHÌA KHÓA MƯỢT

        # ---------------- Horizontal ----------------
        if self._variant in ("panRight", "panLeft"):
            fill_w = rect.width() * fill_factor
            grad = QLinearGradient(0, 0, fill_w, 0)

            for i, c in enumerate(colors + [colors[0]]):
                grad.setColorAt(i / len(colors), c)

            pos = self._offset
            if self._variant == "panLeft":
                pos = 1 - pos

            offset = -pos * fill_w
            grad.setStart(offset, 0)
            grad.setFinalStop(offset + fill_w, 0)

        # ---------------- Vertical ----------------
        else:
            fill_h = rect.height() * fill_factor
            grad = QLinearGradient(0, 0, 0, fill_h)

            for i, c in enumerate(colors + [colors[0]]):
                grad.setColorAt(i / len(colors), c)

            pos = self._offset
            if self._variant == "panUp":
                pos = 1 - pos

            offset = -pos * fill_h
            grad.setStart(0, offset)
            grad.setFinalStop(0, offset + fill_h)

        grad.setSpread(QLinearGradient.RepeatSpread)

        # DEBUG chỗ nghi ngờ
        # if abs(self._offset) < 0.01 or abs(self._offset - 1) < 0.01:
        #     print(f"[DEBUG] offset={self._offset:.3f}, start={grad.start()}, end={grad.finalStop()}")

        palette = self._widget.palette()
        palette.setBrush(self._widget.backgroundRole(), QBrush(grad))
        self._widget.setAutoFillBackground(True)
        self._widget.setPalette(palette)






class DemoFrame(QFrame):
    """
    Frame demo dùng để:
    - Hiển thị label mô tả
    - Gắn BackgroundPanEffect
    """

    def __init__(self, title: str, effect_kwargs: dict):
        super().__init__()

        # ==========================
        # Style cơ bản cho frame demo
        # ==========================
        self.setFixedHeight(120)
        self.setFrameShape(QFrame.StyledPanel)
        self.setFrameShadow(QFrame.Raised)

        # ==========================
        # Layout + label
        # ==========================
        layout = QVBoxLayout(self)
        layout.setContentsMargins(16, 16, 16, 16)

        label = QLabel(title)
        label.setAlignment(Qt.AlignCenter)
        label.setStyleSheet(
            """
            QLabel {
                color: white;
                font-size: 14px;
                font-weight: bold;
            }
            """
        )

        layout.addStretch()
        layout.addWidget(label)
        layout.addStretch()
        
        self.setStyleSheet("background-color: transparent;border: 1px solid red; border-radius: 16px;")

        # ==========================
        # Apply BackgroundPanEffect
        # ==========================
        self._effect = BackgroundPanEffect(
            widget=self,
            **effect_kwargs
        )


class MainWindow(QMainWindow):
    def __init__(self):
        super().__init__()

        self.setWindowTitle("BackgroundPanEffect Demo")
        self.resize(900, 600)

        # =====================================================
        # Central widget là QFrame (đúng yêu cầu)
        # =====================================================
        central = QFrame()
        central.setObjectName("central")
        central.setStyleSheet(
            """
            QFrame#central {
                background-color: #121212;
            }
            """
        )
        self.setCentralWidget(central)

        # =====================================================
        # Layout chính
        # =====================================================
        root_layout = QVBoxLayout(central)
        root_layout.setSpacing(16)
        root_layout.setContentsMargins(24, 24, 24, 24)

        # =====================================================
        # Case 1: Preset colors + panRight + light theme
        # =====================================================
        root_layout.addWidget(
            DemoFrame(
                title='Preset "ocean" | panRight | light',
                effect_kwargs=dict(
                    variant="panRight",
                    colors="ocean",
                    theme="light",
                ),
            )
        )

        # =====================================================
        # Case 2: Preset colors + panLeft + dark theme
        # =====================================================
        root_layout.addWidget(
            DemoFrame(
                title='Preset "sunset" | panLeft | dark',
                effect_kwargs=dict(
                    variant="panLeft",
                    colors="sunset",
                    theme="dark",
                ),
            )
        )

        # =====================================================
        # Case 3: Custom colors + RGB interpolate
        # =====================================================
        root_layout.addWidget(
            DemoFrame(
                title='Custom colors | RGB | panDown',
                effect_kwargs=dict(
                    variant="panDown",
                    colors=("#ee7752", "#23d5ab"),
                    interpolate="rgb",
                    steps=4,
                    theme="light",
                ),
            )
        )

        # =====================================================
        # Case 4: Custom colors + HSL interpolate (mượt hơn)
        # =====================================================
        root_layout.addWidget(
            DemoFrame(
                title='Custom colors | HSL | panUp',
                effect_kwargs=dict(
                    variant="panUp",
                    colors=("#00c6ff", "#0072ff"),
                    interpolate="hsl",
                    steps=6,
                    theme="dark",
                ),
            )
        )

        # =====================================================
        # Case 5: Forest preset + nhiều steps
        # =====================================================
        root_layout.addWidget(
            DemoFrame(
                title='Preset "forest" | panRight | steps=8',
                effect_kwargs=dict(
                    variant="panRight",
                    colors="forest",
                    interpolate="hsl",
                    steps=8,
                    theme="light",
                ),
            )
        )

        root_layout.addStretch()


if __name__ == "__main__":
    app = QApplication(sys.argv)

    win = MainWindow()
    win.show()

    sys.exit(app.exec())



from typing import Callable
from dataclasses import dataclass



class ResolverType:
    validate: Callable
    errors: dict

@dataclass
class UseFormType:
  handleSubmit: Callable
  setValue: Callable
  formState: dict
  control: dict
  watch: Callable
  resolver: ResolverType




# Copyright (C) 2022 The Pyact Company Ltd.
# SPDX-License-Identifier: LicenseRef-Pyact-Commercial OR BSD-3-Clause
from __future__ import annotations

"""..site_packages.qtcompat port of the areachart example from qtmui v1.0"""

from typing import Optional, Union
import sys
from PySide6.QtCore import QPointF, Qt
from PySide6.QtWidgets import QMainWindow, QApplication, QWidget, QVBoxLayout
from PySide6.QtCharts import (QChart, 
                              QChartView, 
                              QLineSeries, 
                              QAreaSeries,
                              QBarCategoryAxis, 
                              QBarSeries, 
                              QBarSet, 
                              QChart,
                              QChartView, 
                              QValueAxis,
                              )
from PySide6.QtGui import QGradient, QPen, QLinearGradient, QPainter

from .chart_line import ChartLine
from .map_change_theme import ChartArea
from .chart_bar import ChartBar


class Chart(QWidget):
    def __init__(
                self,
                dir: str = "ltr", # "line" | "area" | "bar" | "pie" | "donut" | "radialBar" | "scatter" | "bubble" | "heatmap" | "candlestick" | "boxPlot" | "radar" | "polarArea" | "rangeBar" | "rangeArea" | "treemap"
                type: str = "line", # "line" | "area" | "bar" | "pie" | "donut" | "radialBar" | "scatter" | "bubble" | "heatmap" | "candlestick" | "boxPlot" | "radar" | "polarArea" | "rangeBar" | "rangeArea" | "treemap"
                series: object = None, 
                width: Optional[Union[str, int]] = None, 
                height: Optional[Union[str, int]] = None, 
                options: object = None,
                key: str = None,
                *args,
                **kwargs
                ):
        super().__init__(*args, **kwargs)
        self.setLayout(QVBoxLayout())
        self.layout().setContentsMargins(0,0,0,0)

        if type == "line":
            line_chart = ChartLine(
                type=type,
                series=series,
                width=width,
                height=height,
                options=options
            )
            self.layout().addWidget(line_chart)
        elif type == "area":
            area_chart = ChartArea(
                type=type,
                series=series,
                width=width,
                height=height,
                options=options
            )
            self.layout().addWidget(area_chart)
        elif type == "bar":
            bar_chart = ChartBar(
                type=type,
                series=series,
                width=width,
                height=height,
                options=options
            )
            self.layout().addWidget(bar_chart)

        if height:
             self.setFixedHeight(height)


from typing import Dict, TYPE_CHECKING
if TYPE_CHECKING:
    from qtmui.material.styles.create_theme.theme_reducer import ThemeState

from .properties_name import *
from ....system.color_manipulator import alpha


def paper(_theme) -> Dict:
    theme: ThemeState = _theme
    # theme = _theme
    return {
        'PyPaper': {
            'styles': {
                "root": {
                    # backgroundColor:  theme.palette.background.paper,
                    backgroundColor:  "#ffffff",
                    borderRadius: "8px"
                },
                'outlined': {
                    # borderColor: alpha(theme.palette.grey._500, 0.16),
                    border: "1px solid rgba(0,0,0,0.12)"
                },
            },
        },
    }




from typing import Callable
from qtmui.hooks import State
from qtmui.i18n.use_translation import translate


def getTranslatedText(input):
    if isinstance(input, Callable):
        return translate(input)
    elif isinstance(input, State):
        if isinstance(input.value, Callable):
            return translate(input.value)
        elif isinstance(input.value, str):
            return input.value
    elif isinstance(input, str):
        return input


import asyncio
from functools import lru_cache
from typing import Optional, Union, Callable, Any, List, Dict
import uuid

from PySide6.QtWidgets import QWidget, QVBoxLayout, QFrame, QSizePolicy, QHBoxLayout
from PySide6.QtCore import Qt, QTimer, QRunnable, QThreadPool

from qtmui.material.styles import useTheme
from qtmui.hooks import State, useEffect
from qtmui.material.styles.create_theme.components.get_qss_styles import get_qss_style

from ..utils.validate_params import _validate_param
from ..widget_base import PyWidgetBase

from qtmui.configs import LOAD_WIDGET_ASYNC


class Card(QFrame, PyWidgetBase):
    """
    A component that displays content in a card layout with customizable styling and elevation.

    The `Card` component is used to group content with a paper-like appearance, supporting
    elevation, outlined variants, and custom layouts. It supports all props of the Material-UI
    `Card` and `Paper` components, as well as additional props for direction, full width,
    click events, and spacing. Props of the `Paper` component are supported via `**kwargs`.

    Parameters
    ----------
    children : State, Any, List[Any], or None, optional
        The content of the component, either a single widget or a list of widgets.
        Default is None. Can be a `State` object for dynamic updates.
    classes : State or dict, optional
        Override or extend the styles applied to the component. Default is None.
        Can be a `State` object for dynamic updates.
    component : State or Any, optional
        Component used for the root node (e.g., HTML element or custom component).
        Default is None. Can be a `State` object for dynamic updates.
    direction : State or str, optional
        Direction of the layout ("column" or "row"). Default is "column".
        Can be a `State` object for dynamic updates.
    elevation : State or int, optional
        Shadow depth of the card, from 0 to 24. Default is 1.
        Can be a `State` object for dynamic updates.
    fullWidth : State or bool, optional
        If True, the card expands to full width. Default is True.
        Can be a `State` object for dynamic updates.
    key : State or str, optional
        Unique key for the component, used for internal purposes. Default is None.
        Can be a `State` object for dynamic updates.
    onClick : State or Callable, optional
        Function to call when the card is clicked. Default is None.
        Can be a `State` object for dynamic updates.
    raised : State or bool, optional
        If True, uses raised styling (sets elevation to 4 if not specified).
        Default is False. Can be a `State` object for dynamic updates.
    square : State or bool, optional
        If True, the card has sharp corners (no border-radius). Default is False.
        Can be a `State` object for dynamic updates.
    variant : State or str, optional
        Variant of the card ("elevation" or "outlined"). Default is "elevation".
        Can be a `State` object for dynamic updates.
    spacing : State or int, optional
        Spacing between child elements in pixels. Default is 6.
        Can be a `State` object for dynamic updates.
    sx : State, dict, Callable, str, or None, optional
        System prop for CSS overrides and additional styles. Default is None.
        Can be a `State` object for dynamic updates.
    **kwargs
        Additional keyword arguments passed to the parent `QFrame` class, supporting
        props of the `Paper` component (e.g., style, className).

    Attributes
    ----------
    VALID_VARIANTS : list[str]
        Valid values for the `variant` parameter: ["elevation", "outlined"].
    VALID_DIRECTIONS : list[str]
        Valid values for the `direction` parameter: ["column", "row"].

    Notes
    -----
    - Props of the `Paper` component are supported via `**kwargs` (e.g., `style`, `className`).
    - The `raised` prop sets `elevation` to 4 if `elevation` is not explicitly set.
    - Additional props (`direction`, `fullWidth`, `key`, `onClick`, `spacing`) are specific
      to this implementation and not part of Material-UI `Card` or `Paper`.

    Demos:
    - Card: https://qtmui.com/material-ui/qtmui-card/

    API Reference:
    - Card API: https://qtmui.com/material-ui/api/card/
    - Paper API: https://qtmui.com/material-ui/api/paper/
    """

    VALID_VARIANTS = ["elevation", "outlined"]
    VALID_DIRECTIONS = ["column", "row"]

    def __init__(
        self,
        children: Optional[Union[State, Any, List[Any]]] = None,
        classes: Optional[Union[State, Dict]] = None,
        component: Optional[Union[State, Any]] = None,
        direction: Union[State, str] = "column",
        elevation: Union[State, int] = 1,
        fullWidth: Union[State, bool] = True,
        key: Optional[Union[State, str]] = None,
        onClick: Optional[Union[State, Callable]] = None,
        raised: Union[State, bool] = False,
        square: Union[State, bool] = False,
        variant: Union[State, str] = "elevation",
        spacing: Union[State, int] = 6,
        sizePolicy: QSizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Maximum),
        sx: Optional[Union[State, Dict, Callable, str]] = None,
        **kwargs
    ):
        super().__init__()
        
        self.setObjectName(str(uuid.uuid4()))
        
        PyWidgetBase._setUpUi(self)

        self.theme = useTheme()

        # Set properties with validation
        self._set_children(children)
        self._set_classes(classes)
        self._set_component(component)
        self._set_direction(direction)
        self._set_elevation(elevation)
        self._set_fullWidth(fullWidth)
        self._set_key(key)
        self._set_onClick(onClick)
        self._set_raised(raised)
        self._set_square(square)
        self._set_variant(variant)
        self._set_spacing(spacing)
        self._set_sx(sx)
        
        self._sizePolicy = sizePolicy

        self._init_ui()


    # Setter and Getter methods for all parameters
    # @_validate_param(file_path="qtmui.material.card", param_name="children", supported_signatures=Union[State, Any, List[Any], type(None)])
    def _set_children(self, value):
        self._children = value

    def _get_children(self):
        return self._children.value if isinstance(self._children, State) else self._children

    @_validate_param(file_path="qtmui.material.card", param_name="classes", supported_signatures=Union[State, Dict, type(None)])
    def _set_classes(self, value):
        self._classes = value

    def _get_classes(self):
        return self._classes.value if isinstance(self._classes, State) else self._classes

    # @_validate_param(file_path="qtmui.material.card", param_name="component", supported_signatures=Union[State, Any, type(None)])
    def _set_component(self, value):
        self._component = value

    def _get_component(self):
        return self._component.value if isinstance(self._component, State) else self._component

    @_validate_param(file_path="qtmui.material.card", param_name="direction", supported_signatures=Union[State, str], valid_values=VALID_DIRECTIONS)
    def _set_direction(self, value):
        self._direction = value

    def _get_direction(self):
        return self._direction.value if isinstance(self._direction, State) else self._direction

    @_validate_param(file_path="qtmui.material.card", param_name="elevation", supported_signatures=Union[State, int])
    def _set_elevation(self, value):
        if not (0 <= value <= 24):
            raise ValueError("elevation must be between 0 and 24")
        self._elevation = value

    def _get_elevation(self):
        return self._elevation.value if isinstance(self._elevation, State) else self._elevation

    @_validate_param(file_path="qtmui.material.card", param_name="fullWidth", supported_signatures=Union[State, bool])
    def _set_fullWidth(self, value):
        self._fullWidth = value

    def _get_fullWidth(self):
        return self._fullWidth.value if isinstance(self._fullWidth, State) else self._fullWidth

    @_validate_param(file_path="qtmui.material.card", param_name="key", supported_signatures=Union[State, str, type(None)])
    def _set_key(self, value):
        self._key = value

    def _get_key(self):
        return self._key.value if isinstance(self._key, State) else self._key

    @_validate_param(file_path="qtmui.material.card", param_name="onClick", supported_signatures=Union[State, Callable, type(None)])
    def _set_onClick(self, value):
        self._onClick = value

    def _get_onClick(self):
        return self._onClick.value if isinstance(self._onClick, State) else self._onClick

    @_validate_param(file_path="qtmui.material.card", param_name="raised", supported_signatures=Union[State, bool])
    def _set_raised(self, value):
        self._raised = value

    def _get_raised(self):
        return self._raised.value if isinstance(self._raised, State) else self._raised

    @_validate_param(file_path="qtmui.material.card", param_name="square", supported_signatures=Union[State, bool])
    def _set_square(self, value):
        self._square = value

    def _get_square(self):
        return self._square.value if isinstance(self._square, State) else self._square

    @_validate_param(file_path="qtmui.material.card", param_name="variant", supported_signatures=Union[State, str], valid_values=VALID_VARIANTS)
    def _set_variant(self, value):
        self._variant = value

    def _get_variant(self):
        return self._variant.value if isinstance(self._variant, State) else self._variant

    @_validate_param(file_path="qtmui.material.card", param_name="spacing", supported_signatures=Union[State, int])
    def _set_spacing(self, value):
        if value < 0:
            raise ValueError("spacing must be non-negative")
        self._spacing = value

    def _get_spacing(self):
        return self._spacing.value if isinstance(self._spacing, State) else self._spacing

    @_validate_param(file_path="qtmui.material.card", param_name="sx", supported_signatures=Union[State, Dict, Callable, str, type(None)])
    def _set_sx(self, value):
        self._sx = value

    def _get_sx(self):
        return self._sx.value if isinstance(self._sx, State) else self._sx


    def _init_ui(self):

        if self._direction == "column":
            self.setLayout(QVBoxLayout())
            self.layout().setAlignment(Qt.AlignmentFlag.AlignTop)
        else:
            self.setLayout(QHBoxLayout())
        
        if isinstance(self._sizePolicy, QSizePolicy):
            self.setSizePolicy(self._sizePolicy)

        if self._onClick:
            self.setCursor(Qt.CursorShape.PointingHandCursor)

        self.layout().setContentsMargins(0,0,0,0)
        self.layout().setSpacing(self._spacing)

        # Xử lý phần tử con
        if self._children:
            if isinstance(self._children, list):
                for child in self._children:
                    if child is not None:
                        if LOAD_WIDGET_ASYNC:
                            self._do_task_async(lambda child=child: self.layout().addWidget(child))
                        else:
                            self.layout().addWidget(child)

        useEffect(
            self._set_stylesheet,
            [self.theme.state]
        )
        self._set_stylesheet()

    @classmethod
    @lru_cache(maxsize=128)
    def _get_stylesheet(cls, _theme_mode: str):
        
        theme = useTheme()
        PyCard_root = theme.components[f"PyCard"].get("styles")["root"]
        PyCard_root_qss = get_qss_style(PyCard_root)

        _________object_name_______ = "_________object_name_______"

        stylesheet = f"""
            #{_________object_name_______}{{
                {PyCard_root_qss}
            }}
        """

        return stylesheet

    def _set_stylesheet(self, component_styled=None):
        _theme_mode = useTheme().palette.mode
        stylesheet = self._get_stylesheet(_theme_mode)
        stylesheet = stylesheet.replace("_________object_name_______", self.objectName())

        sx_qss = ""

        if self._sx:
            if isinstance(self._sx, dict):
                sx_qss = get_qss_style(self._sx, class_name=f"#{self.objectName()}")
            elif isinstance(self._sx, Callable):
                sx = self._sx()
                if isinstance(sx, dict):
                    sx_qss = get_qss_style(sx, class_name=f"#{self.objectName()}")
                elif isinstance(sx, str):
                    sx_qss = sx
            elif isinstance(self._sx, str) and self._sx != "":
                sx_qss = self._sx

        self.setStyleSheet(stylesheet + sx_qss)


    def mouseReleaseEvent(self, event):
        if self._onClick:
            self._onClick()
        return super().mouseReleaseEvent(event)

    def paintEvent(self, arg__1):
        PyWidgetBase.paintEvent(self, arg__1)
        return super().paintEvent(arg__1)

    def resizeEvent(self, event):
        PyWidgetBase.resizeEvent(self, event)
        return super().resizeEvent(event)


import uuid
from typing import Callable, Optional
from PySide6.QtWidgets import QFrame, QPushButton, QMenu
from PySide6.QtCore import QPoint, Qt, QEvent, QSize, QCoreApplication, QTimer
from PySide6.QtGui import QMouseEvent, QIcon, QCursor

from ...common.icon import FluentIconBase
from .button import Button


# https://stackoverflow.com/questions/37797342/pyqt5-mousetracking-not-working


class MenuButton(Button):
    def __init__(
                self,
                context=None,
                menu: Optional[any] = None,
                icon: str = None,
                aboutToHide: Callable = None,
                *args, **kwargs
                ):
        super().__init__(startIcon=icon or ":/round/resource_qtmui/round/more_vert.svg", *args, **kwargs)
        self._context = context
        self._menu = menu

        self._icon = icon
        self._aboutToHide = aboutToHide

        self.is_open = False
        self.setMouseTracking(True)

        # self.setMenu(menu) # xem lai
        # self.menu = menu
        if aboutToHide:
            self.menu.aboutToHide.connect(self._aboutToHide)
        self.setContextMenuPolicy(Qt.CustomContextMenu)

        self.setStyleSheet('''
            QPushButton::menu-indicator {
                image: none;
            }
        ''')

    # def onMenuHide(self):
    #     QTimer.singleShot(500, self.updateUiOnMenuHide)


    def updateUiOnMenuHide(self):
        if not self._icon:
            self.update_state(False)
        self.is_open = False

    def update_state(self, is_open):
      if self._icon is not None:
        if is_open:
            self.setStyleSheet('''
            QPushButton {
                background-origin: padding;
                background-image: url(":/icons/svg_cache/ri_arrow-up-s-line.svg");
                background-position: right center;
                background-repeat: no-repeat;
                border: 0px solid transparent !important;
                border-right: 5px solid transparent;
                border-bottom: 5px solid transparent;
                border-radius: 4px;
                color: #6b6b6b;
                font-weight: 600;
                font-size: 12px;
            }
            QPushButton:hover {background-color: transparent;}
            ''')
        else:
            self.setStyleSheet('''
            QPushButton {
                background-origin: padding;
                background-image: url(":/icons/svg_cache/ri_arrow-down-s-line.svg");
                background-position: right center;
                background-repeat: no-repeat;
                border: 0px solid transparent !important;
                border-right: 5px solid transparent;
                border-bottom: 5px solid transparent;
                border-radius: 4px;
                color: #6b6b6b;
                font-weight: 600;
                font-size: 12px;
            }
            QPushButton:hover {background-color: transparent;}
            ''')


    def open_context_menu(self, point=None):
        if self.is_open == False:
          self.is_open = True
        #   self.update_state(True)
          self.menu().hide()
          point = QPoint(40, - self.get_menu_height() + self.height())
          self.menu().exec(self.mapToGlobal(point))
        else:
          self.menu().hide()
        #   self.update_state(False)
          self.is_open = False

    def get_menu_height(self):
        # Hiển thị menu tại vị trí (-1000, -1000) để nó không hiển thị trên màn hình
        self.menu().popup(self.mapToGlobal(self.pos()) + self.rect().bottomRight() + QPoint(-1000, -1000))
        # Lấy kích thước của menu sau khi nó được hiển thị
        height = self.menu().height()
        return height


import os
import sys
from typing import Any, Callable, Dict, Optional, Union
import uuid

from PySide6.QtWidgets import QTextEdit, QMainWindow, QVBoxLayout, QFrame, QStatusBar, QToolBar, QFileDialog, QComboBox, QFontComboBox, QMessageBox
from PySide6.QtCore import QSize, Qt
from PySide6.QtGui import QImage, QTextDocument, QAction, QFont, QKeySequence, QActionGroup
from PySide6.QtPrintSupport import QPrintDialog


from ..system.color_manipulator import alpha

from qtmui.material.styles.create_theme.components.get_qss_styles import get_qss_style

from qtmui.hooks import useEffect
from qtmui.material.styles import useTheme
from qtmui.i18n.use_translation import translate, i18n

from ..py_iconify import PyIconify
from ...qtmui_assets import QTMUI_ASSETS

FONT_SIZES = [7, 8, 9, 10, 11, 12, 13, 14, 18, 24, 36, 48, 64, 72, 96, 144, 288]
IMAGE_EXTENSIONS = ['.jpg','.png','.bmp']
HTML_EXTENSIONS = ['.htm', '.html']

def hexuuid():
    return uuid.uuid4().hex

def splitext(p):
    return os.path.splitext(p)[1].lower()

class TextEdit(QTextEdit):

    def canInsertFromMimeData(self, source):

        if source.hasImage():
            return True
        else:
            return super(TextEdit, self).canInsertFromMimeData(source)

    def insertFromMimeData(self, source):

        cursor = self.textCursor()
        document = self.document()

        if source.hasUrls():

            for u in source.urls():
                file_ext = splitext(str(u.toLocalFile()))
                if u.isLocalFile() and file_ext in IMAGE_EXTENSIONS:
                    image = QImage(u.toLocalFile())
                    document.addResource(QTextDocument.ImageResource, u, image)
                    cursor.insertImage(u.toLocalFile())

                else:
                    # If we hit a non-image or non-local URL break the loop and fall out
                    # to the super call & let Qt handle it
                    break

            else:
                # If all were valid images, finish here.
                return


        elif source.hasImage():
            image = source.imageData()
            uuid = hexuuid()
            document.addResource(QTextDocument.ImageResource, uuid, image)
            cursor.insertImage(uuid)
            return

        super(TextEdit, self).insertFromMimeData(source)


class Editor(QMainWindow):

    def __init__(
            self, 
            id: Optional[str] = None, 
            placeholder: Optional[str] = None, 
            value: Optional[Any] = None, 
            onChange: Optional[Callable] = None, 
            simple: Optional[bool] = None, 
            sx: Optional[Union[Callable, str, Dict]]= None,
            *args, 
            **kwargs
        ):
        super(Editor, self).__init__(*args, **kwargs)

        self.onChange = onChange
        self.simple = simple

        self.setObjectName(str(uuid.uuid4()))

        layout = QVBoxLayout()
        layout.setContentsMargins(2,2,2,2)
        self.editor = TextEdit()
        self.editor.setPlaceholderText("Write something awesome...")
        self.editor.setObjectName(str(uuid.uuid4()))
        # Setup the QTextEdit editor configuration
        self.editor.setAutoFormatting(QTextEdit.AutoAll)
        self.editor.selectionChanged.connect(self.update_format)
        # Initialize default font size.
        font = QFont('Times', 12)
        self.editor.setFont(font)
        # We need to repeat the size to init the current format.
        self.editor.setFontPointSize(12)

        # self.path holds the path of the currently open file.
        # If none, we haven't got a file open yet (or creating new).
        self.path = None

        layout.addWidget(self.editor)

        self.container = QFrame(self)
        self.container.setObjectName(str(uuid.uuid4()))
        self.container.setLayout(layout)
        self.setCentralWidget(self.container)

        self.status = QStatusBar()
        # self.setStatusBar(self.status)

        # Uncomment to disable native menubar on Mac
        # self.menuBar().setNativeMenuBar(False)

        file_toolbar = QToolBar("Fileee")
        file_toolbar.setIconSize(QSize(14, 14))
        self.addToolBar(file_toolbar)
        # file_menu = self.menuBar().addMenu("&Fileee")

        open_file_action = QAction(PyIconify(key=QTMUI_ASSETS.ICONS.FILE_OPEN), "Open file...", self)

        open_file_action.setStatusTip("Open file")
        open_file_action.triggered.connect(self.file_open)
        # file_menu.addAction(open_file_action)
        file_toolbar.addAction(open_file_action)

        save_file_action = QAction(PyIconify(key=QTMUI_ASSETS.ICONS.FILE_SAVE), "Save", self)

        save_file_action.setStatusTip("Save current page")
        save_file_action.triggered.connect(self.file_save)
        # file_menu.addAction(save_file_action)
        file_toolbar.addAction(save_file_action)

        saveas_file_action = QAction(PyIconify(key=QTMUI_ASSETS.ICONS.FILE_SAVE_AS), "Save As...", self)
        saveas_file_action.setStatusTip("Save current page to specified file")
        saveas_file_action.triggered.connect(self.file_saveas)
        # file_menu.addAction(saveas_file_action)
        file_toolbar.addAction(saveas_file_action)

        print_action = QAction(PyIconify(key=QTMUI_ASSETS.ICONS.PRINT), "Print...", self)
        print_action.setStatusTip("Print current page")
        print_action.triggered.connect(self.file_print)
        # file_menu.addAction(print_action)
        file_toolbar.addAction(print_action)

        edit_toolbar = QToolBar("Edit")
        edit_toolbar.setIconSize(QSize(16, 16))
        self.addToolBar(edit_toolbar)
        # edit_menu = self.menuBar().addMenu("&Edit")

        undo_action = QAction(PyIconify(key=QTMUI_ASSETS.ICONS.UNDO), "Undo", self)
        undo_action.setStatusTip("Undo last change")
        undo_action.triggered.connect(self.editor.undo)
        # edit_menu.addAction(undo_action)

        redo_action = QAction(PyIconify(key=QTMUI_ASSETS.ICONS.REDO), "Redo", self)
        redo_action.setStatusTip("Redo last change")
        redo_action.triggered.connect(self.editor.redo)
        edit_toolbar.addAction(redo_action)
        # edit_menu.addAction(redo_action)

        # edit_menu.addSeparator()

        cut_action = QAction(PyIconify(key=QTMUI_ASSETS.ICONS.CONTENT_CUT), "Cut", self)
        cut_action.setStatusTip("Cut selected text")
        cut_action.setShortcut(QKeySequence.Cut)
        cut_action.triggered.connect(self.editor.cut)
        edit_toolbar.addAction(cut_action)
        # edit_menu.addAction(cut_action)

        copy_action = QAction(PyIconify(key=QTMUI_ASSETS.ICONS.CONTENT_COPY), "Copy", self)
        copy_action.setStatusTip("Copy selected text")
        cut_action.setShortcut(QKeySequence.Copy)
        copy_action.triggered.connect(self.editor.copy)
        edit_toolbar.addAction(copy_action)
        # edit_menu.addAction(copy_action)

        paste_action = QAction(PyIconify(key=QTMUI_ASSETS.ICONS.CONTENT_PASTE), "Paste", self)
        paste_action.setStatusTip("Paste from clipboard")
        cut_action.setShortcut(QKeySequence.Paste)
        paste_action.triggered.connect(self.editor.paste)
        edit_toolbar.addAction(paste_action)
        # edit_menu.addAction(paste_action)

        select_action = QAction(PyIconify(key=QTMUI_ASSETS.ICONS.SELECT_ALL), "Select all", self)
        select_action.setStatusTip("Select all text")
        cut_action.setShortcut(QKeySequence.SelectAll)
        select_action.triggered.connect(self.editor.selectAll)
        # edit_menu.addAction(select_action)

        # edit_menu.addSeparator()

        wrap_action = QAction(PyIconify(key=QTMUI_ASSETS.ICONS.WRAP_TEXT), "Wrap text to window", self)
        wrap_action.setStatusTip("Toggle wrap text to window")
        wrap_action.setCheckable(True)
        wrap_action.setChecked(True)
        wrap_action.triggered.connect(self.edit_toggle_wrap)
        # edit_menu.addAction(wrap_action)

        format_toolbar = QToolBar("Format")
        format_toolbar.setIconSize(QSize(16, 16))
        self.addToolBar(format_toolbar)
        # format_menu = self.menuBar().addMenu("&Format")

        # We need references to these actions/settings to update as selection changes, so attach to self.
        self.fonts = QFontComboBox(self)
        self.fonts.currentFontChanged.connect(self.editor.setCurrentFont)
        format_toolbar.addWidget(self.fonts)

        self.fontsize = QComboBox(self)
        self.fontsize.addItems([str(s) for s in FONT_SIZES])

        # Connect to the signal producing the text of the current selection. Convert the string to float
        # and set as the pointsize. We could also use the index + retrieve from FONT_SIZES.
        # self.fontsize.currentIndexChanged[str].connect(lambda s: self.editor.setFontPointSize(float(s)) )
        self.fontsize.currentIndexChanged.connect(lambda i: self.editor.setFontPointSize(float(self.fontsize.itemText(i))))

        format_toolbar.addWidget(self.fontsize)

        self.bold_action = QAction(PyIconify(key=QTMUI_ASSETS.ICONS.FORMAT_BOLD), "Bold", self)
        self.bold_action.setStatusTip("Bold")
        self.bold_action.setShortcut(QKeySequence.Bold)
        self.bold_action.setCheckable(True)
        self.bold_action.toggled.connect(lambda x: self.editor.setFontWeight(QFont.Bold if x else QFont.Normal))
        format_toolbar.addAction(self.bold_action)
        # format_menu.addAction(self.bold_action)

        self.italic_action = QAction(PyIconify(key=QTMUI_ASSETS.ICONS.FORMAT_ITALIC), "Italic", self)
        self.italic_action.setStatusTip("Italic")
        self.italic_action.setShortcut(QKeySequence.Italic)
        self.italic_action.setCheckable(True)
        self.italic_action.toggled.connect(self.editor.setFontItalic)
        format_toolbar.addAction(self.italic_action)
        # format_menu.addAction(self.italic_action)

        self.underline_action = QAction(PyIconify(key=QTMUI_ASSETS.ICONS.FORMAT_UNDERLINED), "Underline", self)
        self.underline_action.setStatusTip("Underline")
        self.underline_action.setShortcut(QKeySequence.Underline)
        self.underline_action.setCheckable(True)
        self.underline_action.toggled.connect(self.editor.setFontUnderline)
        format_toolbar.addAction(self.underline_action)
        # format_menu.addAction(self.underline_action)

        # format_menu.addSeparator()

        self.alignl_action = QAction(PyIconify(key=QTMUI_ASSETS.ICONS.FORMAT_ALIGN_LEFT), "Align left", self)
        self.alignl_action.setStatusTip("Align text left")
        self.alignl_action.setCheckable(True)
        self.alignl_action.triggered.connect(lambda: self.editor.setAlignment(Qt.AlignLeft))
        format_toolbar.addAction(self.alignl_action)
        # format_menu.addAction(self.alignl_action)

        self.alignc_action = QAction(PyIconify(key=QTMUI_ASSETS.ICONS.FORMAT_ALIGN_CENTER), "Align center", self)
        self.alignc_action.setStatusTip("Align text center")
        self.alignc_action.setCheckable(True)
        self.alignc_action.triggered.connect(lambda: self.editor.setAlignment(Qt.AlignCenter))
        format_toolbar.addAction(self.alignc_action)
        # format_menu.addAction(self.alignc_action)

        self.alignr_action = QAction(PyIconify(key=QTMUI_ASSETS.ICONS.FORMAT_ALIGN_RIGHT), "Align right", self)
        self.alignr_action.setStatusTip("Align text right")
        self.alignr_action.setCheckable(True)
        self.alignr_action.triggered.connect(lambda: self.editor.setAlignment(Qt.AlignRight))
        format_toolbar.addAction(self.alignr_action)
        # format_menu.addAction(self.alignr_action)

        self.alignj_action = QAction(PyIconify(key=QTMUI_ASSETS.ICONS.FORMAT_ALIGN_JUSTIFY), "Justify", self)
        self.alignj_action.setStatusTip("Justify text")
        self.alignj_action.setCheckable(True)
        self.alignj_action.triggered.connect(lambda: self.editor.setAlignment(Qt.AlignJustify))
        format_toolbar.addAction(self.alignj_action)
        # format_menu.addAction(self.alignj_action)

        format_group = QActionGroup(self)
        format_group.setExclusive(True)
        format_group.addAction(self.alignl_action)
        format_group.addAction(self.alignc_action)
        format_group.addAction(self.alignr_action)
        format_group.addAction(self.alignj_action)

        # format_menu.addSeparator()

        # A list of all format-related widgets/actions, so we can disable/enable signals when updating.
        self._format_actions = [
            self.fonts,
            self.fontsize,
            self.bold_action,
            self.italic_action,
            self.underline_action,
            # We don't need to disable signals for alignment, as they are paragraph-wide.
        ]

        # Initialize.
        self.update_format()
        self.update_title()
        # self.show()


        theme = useTheme()
        useEffect(
            self._set_stylesheet,
            [theme.state]
        )
        self._set_stylesheet()

    def _set_stylesheet(self):
        theme = useTheme()
        component_styles = theme.components

        # PyEditor_root = component_styles[f"PyEditor"].get("styles")["root"]
        # PyEditor_root_qss = get_qss_style(PyEditor_root)

        # PyEditor
        # self.setStyleSheet(
        #     f"""
        #         #{self.objectName()} {{
        #             border: 1px solid {theme.palette.grey._500};
        #             border-radius: 8px;
        #         }}
        #     """
        # )

        self.setStyleSheet(
            f"""
                #{self.objectName()} {{
                    border: 1px solid {alpha(theme.palette.grey._500, 0.24)};
                    border-radius: 4px;
                }}

                QToolBar {{
                    border-left: 8px solid transparent;
                }}

                QComboBox {{
                    border: none;
                    background-color: transparent;
                    border-radius: 0px;
                    color: {theme.palette.text.primary};
                }}

                QComboBox:hover {{
                    border: none;
                    background-color: transparent;
                }}

                QComboBox QAbstractItemView {{
                    color: {theme.palette.text.primary};
                    selection-color: {theme.palette.action.selected};
                    background-color:  {theme.palette.background.paper};
                    selection-background-color: {theme.palette.action.selected};
                }}

                QComboBox:item:selected {{
                    background-color: {theme.palette.primary.main};
                    color: {theme.palette.text.primary};
                }}

            """
        )

        self.editor.setStyleSheet(
            f"""
                #{self.editor.objectName()} {{
                    border-top: 1px solid {alpha(theme.palette.grey._500, 0.24)};
                    background-color: {theme.palette.background.paper};
                    color: {theme.palette.text.secondary};
                    font-size: 12px;
                    font-weight: {theme.typography.body2.fontWeight};
                    line-height: {theme.typography.body2.lineHeight};
                }}
            """
        )


    def block_signals(self, objects, b):
        for o in objects:
            o.blockSignals(b)

    def update_format(self):
        """
        Update the font format toolbar/actions when a new text selection is made. This is neccessary to keep
        toolbars/etc. in sync with the current edit state.
        :return:
        """
        # Disable signals for all format widgets, so changing values here does not trigger further formatting.
        self.block_signals(self._format_actions, True)

        self.fonts.setCurrentFont(self.editor.currentFont())
        # Nasty, but we get the font-size as a float but want it was an int
        self.fontsize.setCurrentText(str(int(self.editor.fontPointSize())))

        self.italic_action.setChecked(self.editor.fontItalic())
        self.underline_action.setChecked(self.editor.fontUnderline())
        self.bold_action.setChecked(self.editor.fontWeight() == QFont.Bold)

        self.alignl_action.setChecked(self.editor.alignment() == Qt.AlignLeft)
        self.alignc_action.setChecked(self.editor.alignment() == Qt.AlignCenter)
        self.alignr_action.setChecked(self.editor.alignment() == Qt.AlignRight)
        self.alignj_action.setChecked(self.editor.alignment() == Qt.AlignJustify)

        self.block_signals(self._format_actions, False)

    def dialog_critical(self, s):
        dlg = QMessageBox(self)
        dlg.setText(s)
        dlg.setIcon(QMessageBox.Critical)
        dlg.show()

    def file_open(self):
        path, _ = QFileDialog.getOpenFileName(self, "Open file", "", "HTML documents (*.html);Text documents (*.txt);All files (*.*)")

        try:
            with open(path, 'rU') as f:
                text = f.read()

        except Exception as e:
            self.dialog_critical(str(e))

        else:
            self.path = path
            # Qt will automatically try and guess the format as txt/html
            self.editor.setText(text)
            self.update_title()

    def file_save(self):
        if self.path is None:
            # If we do not have a path, we need to use Save As.
            return self.file_saveas()

        text = self.editor.toHtml() if splitext(self.path) in HTML_EXTENSIONS else self.editor.toPlainText()

        try:
            with open(self.path, 'w') as f:
                f.write(text)

        except Exception as e:
            self.dialog_critical(str(e))

    def file_saveas(self):
        path, _ = QFileDialog.getSaveFileName(self, "Save file", "", "HTML documents (*.html);Text documents (*.txt);All files (*.*)")

        if not path:
            # If dialog is cancelled, will return ''
            return

        text = self.editor.toHtml() if splitext(path) in HTML_EXTENSIONS else self.editor.toPlainText()

        try:
            with open(path, 'w') as f:
                f.write(text)

        except Exception as e:
            self.dialog_critical(str(e))

        else:
            self.path = path
            self.update_title()

    def file_print(self):
        dlg = QPrintDialog()
        if dlg.exec_():
            self.editor.print_(dlg.printer())

    def update_title(self):
        self.setWindowTitle("%s - Megasolid Idiom" % (os.path.basename(self.path) if self.path else "Untitled"))

    def edit_toggle_wrap(self):
        self.editor.setLineWrapMode( 1 if self.editor.lineWrapMode() == 0 else 0 )





from .date_time_picker import DateTimePicker
from .time_picker import TimePicker, AMTimePicker
from .date_picker import DatePicker, ZhDatePicker
from .calendar_picker import CalendarPicker
from .color_picker import ColorPicker


from typing import Optional, Union, Dict, Callable, List
import uuid

from ..system.color_manipulator import alpha
from PySide6.QtWidgets import QFrame, QSizePolicy, QWidget, QVBoxLayout, QHBoxLayout
from PySide6.QtCore import Qt
from qtmui.hooks import State
from qtmui.material.styles import useTheme
from qtmui.material.styles.create_theme.components.get_qss_styles import get_qss_style
from ..widget_base import PyWidgetBase
from ..utils.validate_params import _validate_param

class Divider(QFrame, PyWidgetBase):
    """
    A component that renders a divider line, either horizontal or vertical.

    The `Divider` component is used to separate content, supporting all props of the
    Material-UI `Divider` component, as well as additional custom props.

    Parameters
    ----------
    absolute : State or bool, optional
        If True, absolutely positions the divider. Default is False.
        Can be a `State` object for dynamic updates.
    children : State, QWidget, List[QWidget], or None, optional
        The content of the component, typically text or sub-components. Default is None.
        Can be a `State` object for dynamic updates.
    classes : State or dict, optional
        Override or extend the styles applied to the component. Default is None.
        Can be a `State` object for dynamic updates.
    component : State, str, or None, optional
        The component used for the root node (e.g., "QFrame"). Default is None.
        Can be a `State` object for dynamic updates.
    flexItem : State or bool, optional
        If True, adjusts height for vertical dividers in flex containers. Default is False.
        Can be a `State` object for dynamic updates.
    light : State or bool, optional
        If True, uses a lighter color. Default is False.
        Deprecated: Use `sx={{ opacity: 0.6 }}` instead.
        Can be a `State` object for dynamic updates.
    orientation : State or str, optional
        The divider orientation ("horizontal" or "vertical"). Default is "horizontal".
        Can be a `State` object for dynamic updates.
    sx : State, dict, Callable, str, or None, optional
        System prop for CSS overrides and additional styles. Default is None.
        Can be a `State` object for dynamic updates.
    textAlign : State, str, or None, optional
        Text alignment when children are present ("center", "left", "right"). Default is "center".
        Can be a `State` object for dynamic updates.
    variant : State, str, or None, optional
        The divider variant ("fullWidth", "inset", "middle"). Default is "fullWidth".
        Can be a `State` object for dynamic updates.
    color : State, str, or None, optional
        Background color of the divider (custom feature, not part of Material-UI). Default is None.
        Can be a `State` object for dynamic updates.
    height : State or int, optional
        Fixed height for horizontal dividers (custom feature, not part of Material-UI). Default is 1.
        Can be a `State` object for dynamic updates.
    width : State or int, optional
        Fixed width for vertical dividers (custom feature, not part of Material-UI). Default is 1.
        Can be a `State` object for dynamic updates.
    **kwargs
        Additional keyword arguments passed to the parent `QFrame` class, supporting
        props of the native component (e.g., style, className).

    Attributes
    ----------
    VALID_ORIENTATION : list[str]
        Valid values for `orientation`: ["horizontal", "vertical"].
    VALID_TEXT_ALIGN : list[str]
        Valid values for `textAlign`: ["center", "left", "right"].
    VALID_VARIANT : list[str]
        Valid values for `variant`: ["fullWidth", "inset", "middle"].

    Notes
    -----
    - Props of the native component are supported via `**kwargs` (e.g., `style`, `className`).
    - The `color`, `height`, and `width` parameters are custom features, not part of Material-UI's `Divider`.
    - The `light` prop is deprecated; use `sx={{ opacity: 0.6 }}` for similar effects.
    - The `children` prop must be a `QWidget`, a list of `QWidget` instances, or a `State` object.

    Demos:
    - Divider: https://qtmui.com/material-ui/qtmui-divider/

    API Reference:
    - Divider API: https://qtmui.com/material-ui/api/divider/
    """

    VALID_ORIENTATION = ["horizontal", "vertical"]
    VALID_TEXT_ALIGN = ["center", "left", "right"]
    VALID_VARIANT = ["fullWidth", "inset", "middle"]

    def __init__(
        self,
        absolute: Union[State, bool] = False,
        children: Optional[Union[State, QWidget, List[QWidget]]] = None,
        classes: Optional[Union[State, Dict]] = None,
        component: Optional[Union[State, str]] = None,
        flexItem: Union[State, bool] = False,
        light: Union[State, bool] = False,
        orientation: Union[State, str] = "horizontal",
        sx: Optional[Union[State, Dict, Callable, str]] = None,
        textAlign: Optional[Union[State, str]] = "center",
        variant: Optional[Union[State, str]] = "fullWidth",
        color: Optional[Union[State, str]] = None,
        height: Union[State, int] = 1,
        width: Union[State, int] = 1,
        **kwargs
    ):
        super().__init__(**kwargs)
        self.setObjectName(str(uuid.uuid4()))

        # Initialize theme
        self.theme = useTheme()

        # Store widget references to prevent Qt deletion
        self._widget_references = []

        # Set properties with validation
        self._set_absolute(absolute)
        self._set_children(children)
        self._set_classes(classes)
        self._set_component(component)
        self._set_flexItem(flexItem)
        self._set_light(light)
        self._set_orientation(orientation)
        self._set_sx(sx)
        self._set_textAlign(textAlign)
        self._set_variant(variant)
        self._set_color(color)
        self._set_height(height)
        self._set_width(width)

        # Setup UI
        self._init_ui()


    # Setter and Getter methods
    @_validate_param(file_path="qtmui.material.divider", param_name="absolute", supported_signatures=Union[State, bool])
    def _set_absolute(self, value):
        """Assign value to absolute."""
        self._absolute = value

    def _get_absolute(self):
        """Get the absolute value."""
        return self._absolute.value if isinstance(self._absolute, State) else self._absolute

    # @_validate_param(file_path="qtmui.material.divider", param_name="children", supported_signatures=Union[State, QWidget, List[QWidget], type(None)])
    def _set_children(self, value):
        """Assign value to children and store widget references."""
        self._widget_references.clear()
        self._children = value
        children = self._get_children()

        if isinstance(children, QWidget):
            self._widget_references.append(children)
            children.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Maximum)
        elif isinstance(children, list):
            for child in children:
                if child is None:
                    continue
                if isinstance(child, QWidget):
                    self._widget_references.append(child)
                    child.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Maximum)
                else:
                    raise TypeError(f"Each element in children must be a QWidget, but got {type(child)}")
        elif children is not None:
            raise TypeError(f"children must be a State, QWidget, or list of QWidgets, but got {type(children)}")

    def _get_children(self):
        """Get the children value."""
        return self._children.value if isinstance(self._children, State) else self._children

    @_validate_param(file_path="qtmui.material.divider", param_name="classes", supported_signatures=Union[State, Dict, type(None)])
    def _set_classes(self, value):
        """Assign value to classes."""
        self._classes = value

    def _get_classes(self):
        """Get the classes value."""
        return self._classes.value if isinstance(self._classes, State) else self._classes

    @_validate_param(file_path="qtmui.material.divider", param_name="component", supported_signatures=Union[State, str, type(None)])
    def _set_component(self, value):
        """Assign value to component."""
        self._component = value

    def _get_component(self):
        """Get the component value."""
        return self._component.value if isinstance(self._component, State) else self._component

    @_validate_param(file_path="qtmui.material.divider", param_name="flexItem", supported_signatures=Union[State, bool])
    def _set_flexItem(self, value):
        """Assign value to flexItem."""
        self._flexItem = value

    def _get_flexItem(self):
        """Get the flexItem value."""
        return self._flexItem.value if isinstance(self._flexItem, State) else self._flexItem

    @_validate_param(file_path="qtmui.material.divider", param_name="light", supported_signatures=Union[State, bool])
    def _set_light(self, value):
        """Assign value to light."""
        self._light = value

    def _get_light(self):
        """Get the light value."""
        return self._light.value if isinstance(self._light, State) else self._light

    @_validate_param(file_path="qtmui.material.divider", param_name="orientation", supported_signatures=Union[State, str], valid_values=VALID_ORIENTATION)
    def _set_orientation(self, value):
        """Assign value to orientation."""
        self._orientation = value

    def _get_orientation(self):
        """Get the orientation value."""
        return self._orientation.value if isinstance(self._orientation, State) else self._orientation

    @_validate_param(file_path="qtmui.material.divider", param_name="sx", supported_signatures=Union[State, Dict, Callable, str, type(None)])
    def _set_sx(self, value):
        """Assign value to sx."""
        self._sx = value

    def _get_sx(self):
        """Get the sx value."""
        return self._sx.value if isinstance(self._sx, State) else self._sx

    @_validate_param(file_path="qtmui.material.divider", param_name="textAlign", supported_signatures=Union[State, str, type(None)], valid_values=VALID_TEXT_ALIGN)
    def _set_textAlign(self, value):
        """Assign value to textAlign."""
        self._textAlign = value

    def _get_textAlign(self):
        """Get the textAlign value."""
        return self._textAlign.value if isinstance(self._textAlign, State) else self._textAlign

    @_validate_param(file_path="qtmui.material.divider", param_name="variant", supported_signatures=Union[State, str, type(None)], valid_values=VALID_VARIANT)
    def _set_variant(self, value):
        """Assign value to variant."""
        self._variant = value

    def _get_variant(self):
        """Get the variant value."""
        return self._variant.value if isinstance(self._variant, State) else self._variant

    @_validate_param(file_path="qtmui.material.divider", param_name="color", supported_signatures=Union[State, str, type(None)])
    def _set_color(self, value):
        """Assign value to color (custom feature)."""
        self._color = value

    def _get_color(self):
        """Get the color value."""
        return self._color.value if isinstance(self._color, State) else self._color

    @_validate_param(file_path="qtmui.material.divider", param_name="height", supported_signatures=Union[State, int])
    def _set_height(self, value):
        """Assign value to height (custom feature)."""
        self._height = value

    def _get_height(self):
        """Get the height value."""
        return self._height.value if isinstance(self._height, State) else self._height

    @_validate_param(file_path="qtmui.material.divider", param_name="width", supported_signatures=Union[State, int])
    def _set_width(self, value):
        """Assign value to width (custom feature)."""
        self._width = value

    def _get_width(self):
        """Get the width value."""
        return self._width.value if isinstance(self._width, State) else self._width

    def _init_ui(self):

        # self.setFixedSize(QSize(5, 5))

        if self._color is not None:
            if isinstance(self._color, State):
                self._color.valueChanged.connect(self._set_stylesheet)
        else:
            self._color = alpha(self.theme.palette.grey._500, 0.24)

        self.slot_set_stylesheet()
        self.theme.state.valueChanged.connect(self.slot_set_stylesheet)
        self.destroyed.connect(self._on_destroyed)

    def slot_set_stylesheet(self, value=None):
        self._set_stylesheet()


    def _set_stylesheet(self, component_styled=None):
        self.theme = useTheme()

        if not component_styled:
            component_styled = self.theme.components

        sx_qss = ""
        if self._sx:
            if isinstance(self._sx, dict):
                sx_qss = get_qss_style(self._sx, class_name=f"#{self.objectName()}")
            elif isinstance(self._sx, Callable):
                sx = self._sx()
                if isinstance(sx, dict):
                    sx_qss = get_qss_style(sx, class_name=f"#{self.objectName()}")
                elif isinstance(sx, str):
                    sx_qss = sx
            elif isinstance(self._sx, str) and self._sx != "":
                sx_qss = self._sx

        width = ""
        height = ""
        if self._orientation == "horizontal":
            self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Maximum)
            height = f"""
                    min-height: 1px;
                    max-height: 1px;
                    """
        else:
            width = f"""
                    min-width: 1px;
                    max-width: 1px;
                    """  
            self.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Expanding)

        stylesheet = f"""
            Divider {{
                {width}
                {height}
                background-color: {self._color};
            }}

            {sx_qss}

        """

        self.setStyleSheet(stylesheet)




# src/qtmui/material/chart/chart_radar.py
from __future__ import annotations

from typing import List, Dict, Optional, Union, Callable
from PySide6.QtCore import Qt, QPointF
from PySide6.QtWidgets import QWidget, QVBoxLayout
from PySide6.QtCharts import (
    QPolarChart,
    QChartView,
    QLineSeries,
    QAreaSeries,
    QValueAxis,
    QCategoryAxis,
)
from PySide6.QtGui import QPainter, QColor, QBrush, QPen, QFont, QLinearGradient, QGradient

from qtmui.hooks import State
from qtmui.material.styles import useTheme
from qtmui.material.system.color_manipulator import alpha


class ChartRadar(QWidget):
    def __init__(
        self,
        dir: str = "ltr",
        type: str = "radar",
        series: List[Dict] = None,
        options: Optional[Dict] = None,
        height: Optional[Union[int, str]] = None,
        width: Optional[Union[int, str]] = None,
        title: Optional[Union[str, State, Callable]] = None,
        *args,
        **kwargs,
    ):
        super().__init__(*args, **kwargs)
        self.series = series or []
        self.options = options or {}
        self.title = title
        self._height = height
        self._width = width

        self.setLayout(QVBoxLayout())
        self.layout().setContentsMargins(0, 0, 0, 0)
        self.layout().setSpacing(0)

        self.theme = useTheme()
        self.theme.state.valueChanged.connect(self._update_theme)

        self._init_chart()
        self._update_theme()

    def _init_chart(self):
        self.chart = QPolarChart()
        self.chart.setAnimationOptions(QPolarChart.SeriesAnimations)
        self.chart.setBackgroundRoundness(0)

        # Title
        if self.title:
            title_text = self.title() if callable(self.title) else self.title
            self.chart.setTitle(str(title_text))

        # Categories từ options.xaxis.categories
        self.categories = self.options.get("xaxis", {}).get("categories", [])
        if not self.categories and self.series:
            if self.series:
                self.categories = [f"Cat {i+1}" for i in range(len(self.series[0].get("data", [])))]

        # Tính max value động
        all_values = [v for s in self.series for v in s.get("data", []) if isinstance(v, (int, float))]
        self.max_value = max(all_values, default=100)
        self.max_value = self.max_value * 1.1  # +10% padding

        # Axes
        self.angular_axis = self._create_angular_axis()
        self.radial_axis = self._create_radial_axis()

        self.chart.addAxis(self.angular_axis, QPolarChart.PolarOrientationAngular)
        self.chart.addAxis(self.radial_axis, QPolarChart.PolarOrientationRadial)

        # Series + Fill
        self._add_series()

        # Legend cơ bản (không floating phức tạp để tránh crash)
        legend_opts = self.options.get("legend", {})
        legend = self.chart.legend()
        legend.setVisible(legend_opts.get("show", True))
        legend.setAlignment(Qt.AlignBottom)
        legend.show()

        # ChartView
        self.chart_view = QChartView(self.chart)
        self.chart_view.setRenderHint(QPainter.Antialiasing)

        if self._height:
            self.chart_view.setFixedHeight(int(self._height))
        if self._width:
            self.chart_view.setFixedWidth(int(self._width))

        self.layout().addWidget(self.chart_view)

    def _create_angular_axis(self):
        axis = QCategoryAxis()
        n = len(self.categories)
        if n > 0:
            step = 360.0 / n
            for i, label in enumerate(self.categories):
                axis.append(label, i * step)

        axis.setRange(0, 360)

        # Cách gọi đúng, ổn định trên mọi PySide6
        try:
            axis.setLabelsPosition(QCategoryAxis.AxisLabelsPositionOnValue)
        except AttributeError:
            # Fallback cho phiên bản cũ
            axis.setLabelsPosition(1)

        return axis

    def _create_radial_axis(self):
        axis = QValueAxis()
        axis.setRange(0, self.max_value)
        axis.setTickCount(6)
        axis.setLabelFormat("%.0f")
        return axis

    def _add_series(self):
        colors = self.options.get("colors", [])
        if not colors:
            colors = [
                self.theme.palette.primary.main,
                self.theme.palette.warning.main,
                self.theme.palette.info.main,
                self.theme.palette.error.main,
                self.theme.palette.success.main,
            ]

        stroke_opts = self.options.get("stroke", {})
        fill_opts = self.options.get("fill", {})

        stroke_width = stroke_opts.get("width", 3)
        fill_opacity = fill_opts.get("opacity", 0.5)

        n_cats = len(self.categories) or 6
        angle_step = 360.0 / n_cats

        for idx, s in enumerate(self.series):
            name = s.get("name", f"Series {idx+1}")
            data = s.get("data", [])

            upper = QLineSeries()
            upper.setName(name)

            color_str = colors[idx % len(colors)]
            color = QColor(color_str)

            # Fill area (nếu có opacity)
            lower = None
            if fill_opacity > 0:
                lower = QLineSeries()

                fill_color = QColor(alpha(color_str, fill_opacity))
                brush = QBrush(fill_color)

                if fill_opts.get("gradient"):
                    grad = QLinearGradient()
                    grad.setCoordinateMode(QGradient.ObjectBoundingMode)
                    grad.setColorAt(0, QColor(alpha(color_str, fill_opts["gradient"].get("opacityFrom", 0.7))))
                    grad.setColorAt(1, QColor(alpha(color_str, fill_opts["gradient"].get("opacityTo", 0))))
                    brush = QBrush(grad)

                lower.setBrush(brush)

            pen = QPen(color)
            pen.setWidth(stroke_width)
            upper.setPen(pen)

            for i, value in enumerate(data):
                angle = i * angle_step
                radius = (value / self.max_value) * 100 if self.max_value > 0 else 0
                point = QPointF(angle, radius)
                upper.append(point)
                if lower:
                    lower.append(point)

            if lower:
                area = QAreaSeries(lower, upper)
                area.setName(name)
                self.chart.addSeries(area)
                area.attachAxis(self.angular_axis)
                area.attachAxis(self.radial_axis)
            else:
                self.chart.addSeries(upper)
                upper.attachAxis(self.angular_axis)
                upper.attachAxis(self.radial_axis)

    def _update_theme(self):
        theme = useTheme()

        # Background
        bg = QColor(theme.palette.background.paper)
        self.chart.setBackgroundBrush(QBrush(bg))
        self.chart.setPlotAreaBackgroundBrush(QBrush(bg))
        self.chart.setPlotAreaBackgroundVisible(True)

        # Title
        self.chart.setTitleBrush(QBrush(QColor(theme.palette.text.primary)))
        # self.chart.setTitleFont(QFont(theme.typography.h6.fontFamily, 16, QFont.Bold))

        # Grid & labels
        grid_color = QColor(theme.palette.divider)
        text_color = QColor(theme.palette.text.secondary)

        for axis in self.chart.axes():
            axis.setLabelsBrush(QBrush(text_color))
            axis.setLinePen(QPen(grid_color))
            axis.setGridLinePen(QPen(grid_color, 1, Qt.DashLine))

        # Legend
        legend = self.chart.legend()
        legend.setLabelColor(QColor(theme.palette.text.primary))
        # legend.setFont(QFont(theme.typography.body2.fontFamily, 12))


import os
import sys
# os.environ['QT_API'] = 'pyqt5'
from PySide6.QtWidgets import QApplication, QVBoxLayout, QWidget
# from pyqtlet2 import L, MapWidget

class MapWindow(QWidget):
    def __init__(self):
        # Setting up the widgets and layout
        super().__init__()
        # self.mapWidget = MapWidget()
        # self.layout = QVBoxLayout()
        # self.layout.addWidget(self.mapWidget)
        # self.setLayout(self.layout)

        # # Working with the maps with pyqtlet
        # self.map = L.map(self.mapWidget)
        # self.map.setView([12.97, 77.59], 10)
        # L.tileLayer('http://{s}.tile.osm.org/{z}/{x}/{y}.png').addTo(self.map)
        # self.marker = L.marker([12.934056, 77.610029])
        # self.marker.bindPopup('Maps are a treasure.')
        # self.map.addLayer(self.marker)
        # self.show()


from typing import Dict, TYPE_CHECKING
if TYPE_CHECKING:
    from qtmui.material.styles.create_theme.theme_reducer import ThemeState
from qtmui.material.system.color_manipulator import alpha
from .properties_name import *

# from ....css import paper, menuItem
from .paper import paper

def autocomplete(_theme):
    theme: ThemeState = _theme
    theme_palette = theme.palette

    return {
        'MuiAutocomplete': {
            'styles': {
                'root': {
                    "@multiple": {
                        "min-height": 38,
                        "@chip": {
                            "min-height": 44,
                        }
                    }
                    },
                },
                'tag': {
                    # **theme.typography.subtitle2,
                    height: 24,
                    minWidth: 24,
                    lineHeight: '24px',
                    textAlign: 'center',
                    p: theme.spacing(0, 0.75),
                    color: theme_palette.text.secondary,
                    borderRadius: theme.shape.borderRadius,
                    backgroundColor: alpha(theme_palette.grey._500, 0.16),
                },
                'paper': paper({'theme': theme, 'dropdown': True}),
                'listbox': {
                    p: "0px",
                    'option': {
                        # {menuItem(theme)},
                    }
                },
                'endAdornment': {
                    "Iconify": {
                        width: 18,
                        height: 18,
                    },
                },
            }
        }



import uuid
from typing import Optional, Callable, Union, List, Dict
from PySide6.QtWidgets import QWidget, QVBoxLayout, QFrame, QSizePolicy, QCalendarWidget
from PySide6.QtCore import Signal, QSize

from qtmui.material.styles import useTheme
from qtmui.material.styles.create_theme.components.get_qss_styles import get_qss_style

from qtmui.i18n.use_translation import i18n

class DateTimePicker(QCalendarWidget):

    def __init__(
                self,
                anchorEl: QWidget=None
                ):
        super().__init__()
        self.setObjectName(u"calendarview")
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.sizePolicy().hasHeightForWidth())
        self.setSizePolicy(sizePolicy)
        self.setMinimumSize(QSize(0, 300))
        self.setMaximumSize(QSize(320, 300))
        self.setStyleSheet(u"")
        self.setGridVisible(True)
        self.setSelectionMode(QCalendarWidget.SingleSelection)
        self.setHorizontalHeaderFormat(QCalendarWidget.NoHorizontalHeader)
        self.setVerticalHeaderFormat(QCalendarWidget.NoVerticalHeader)
        self.setNavigationBarVisible(True)
        self.setDateEditEnabled(True)

        self._border_style = ""

        self.__init_ui()

        theme = useTheme()
        useEffect(
            self._set_stylesheet,
            [theme.state]
        )
        self._set_stylesheet()

        # i18n.langChanged.connect(self.reTranslation)

    def reTranslation(self):
        pass

    def __init_ui(self):
        pass

    def _set_stylesheet(self):
        theme = useTheme()

        self.setStyleSheet(
            f"""
                    QWidget {{
                        background-color: red; 
                        color:{theme.palette.text.primary};
                        border: solid;
                        border-width: 0px;
                        border-color: transparent;
                        border-radius: 5px;

                    }}

                    QCalendarWidget QToolButton{{
                        height:38px;
                        width:50px;
                        color:white;
                        font:18px 'Segoe UI', 'Microsoft YaHei', 'PingFang SC';
                        icon-size:18px 18px;
                        background-color:#2b2b2b;
                    }}
                    QCalendarWidget QToolButton:hover{{
                        background-color:#202020;
                    }}

                    QCalendarWidget QWidget#qt_calendar_navigationbar{{
                        height:35px;
                        background-color:#2b2b2b;

                    }}


                    #qt_calendar_prevmonth,
                    #qt_calendar_nextmonth{{
                        border:none;
                        qproperty-icon:none;
                        background-color:transparent;
                    }}


                    #qt_calendar_prevmonth{{
                        qproperty-icon: url(:/icons/svg_cache/ri_arrow-left-s-line.svg);
                        icon-size: 15px 15px
                    }}

                    #qt_calendar_nextmonth{{
                        qproperty-icon: url(:/icons/svg_cache/ri_arrow-right-s-line.svg);
                        icon-size: 15px 15px
                    }}


                    #qt_calendar_prevmonth:hover,
                    #qt_calendar_nextmonth:hover{{
                        background-color:#202020;
                    }}




                    #qt_calendar_monthbutton{{
                        width:100px;
                        font:18px 'Segoe UI', 'Microsoft YaHei', 'PingFang SC';
                        color:white;
                        background-color:#2b2b2b;
                        margin: 0px 2px;
                        padding: 0px 2px

                    }}

                    #qt_calendar_yearbutton{{
                        font:18px 'Segoe UI', 'Microsoft YaHei', 'PingFang SC';
                        color:white;
                        background-color:#2b2b2b;
                        margin: 0px 2px;
                        padding: 0px 2px

                    }}

                    #qt_calendar_monthbutton:hover,
                    #qt_calendar_yearbutton:hover{{
                        background-color:#202020;
                    }}

                    /* MenuBar Code */

                    QCalendarWidget QMenu{{
                        width:150px;
                        color:white;
                        font:14px 'Segoe UI', 'Microsoft YaHei', 'PingFang SC';
                        background-color:#2b2b2b;
                    }}

                    /* SpinBox Code */



                    /* Calendar View Code */
                    QCalendarWidget QWidget{{
                        alternate-background-color:#2b2b2b;
                    }}


                    QCalendarWidget QAbstractItemView:enabled{{
                        font:14px 'Segoe UI', 'Microsoft YaHei', 'PingFang SC';
                        color:white;
                        background-color:#2b2b2b;
                        selection-background-color:#202020;
                        selection-color:white;
                    }}
                    QCalendarWidget QAbstractItemView:disabled{{
                        color:rgb(100,100,100);
                    }}

                    QCalendarWidget QSpinBox{{
                        width:60px;
                        height:25px;
                        font:14px 'Segoe UI', 'Microsoft YaHei', 'PingFang SC';
                        color:white;
                        background-color:#2b2b2b;
                    }}

                    #qt_calendar_calendarview{{
                        font:14px 'Segoe UI', 'Microsoft YaHei', 'PingFang SC';
                        color:white;
                        background-color:#2b2b2b;
                        border-width: 0px;
                        border-color: #2b2b2b;
                        border-radius:5px;
                    }}


                    #qt_calendar_calendarview::item{{
                        background-color:#2b2b2b;
                        border-width: 0px;
                        border-color: #2b2b2b;
                        border-radius:25px;

                    }}


                    #qt_calendar_calendarview::item:hover{{
                        background-color:#202020;
                        border-width: 0px;
                        border-color: #202020;
                        border-radius:25px;

                    }}

                    #qt_calendar_calendarview::item:selected{{
                        background-color:black;
                        border-width: 0px;
                        border-color: black;
                        border-radius:25px;
                        
                    }}


            """
        )


from typing import Dict, Any
import contextvars

# Biến toàn cục lưu hook context hiện tại
_current_hook_context = contextvars.ContextVar("current_hook_context")

class HookState:
    def __init__(self):
        self.memo: Dict[int, Any] = {}  # lưu dữ liệu từng hook theo index
        self._hook_index = 0

    def reset(self):
        """Reset hook index về 0 trước mỗi lần render"""
        self._hook_index = 0

    def next_index(self) -> int:
        """Trả index hiện tại, sau đó tăng"""
        index = self._hook_index
        self._hook_index += 1
        return index

    def get(self, index: int) -> Any:
        return self.memo.get(index)

    def set(self, index: int, value: Any):
        self.memo[index] = value

# Hàm lấy hook state hiện tại
def get_hook_state() -> HookState:
    try:
        return _current_hook_context.get()
    except LookupError:
        hs = HookState()
        _current_hook_context.set(hs)
        return hs

# Hàm đặt hook context (gọi ở đầu mỗi lần render component)
def set_current_hook_context(state: HookState):
    _current_hook_context.set(state)



import uuid
from typing import Optional, Union, Dict, List, Callable
from PySide6.QtWidgets import QFrame, QHBoxLayout, QWidget, QVBoxLayout, QLabel
from PySide6.QtCore import Qt
from qtmui.hooks import State
from qtmui.material.styles import useTheme
from ..typography import Typography
from ..utils.validate_params import _validate_param
from qtmui.material.styles.create_theme.components.get_qss_styles import get_qss_style

class ListSubheader(QFrame):
    """
    A component that renders a subheader within a List, styled like Material-UI ListSubheader.

    The `ListSubheader` component is used to display a subheader in a List, with support for custom colors,
    indentation, gutter control, and styling overrides.

    Parameters
    ----------
    children : State, str, QWidget, List[Union[str, QWidget]], or None, optional
        The content of the component. Default is None.
        Can be a `State` object for dynamic updates.
    classes : State or dict, optional
        Override or extend the styles applied to the component. Default is None.
        Can be a `State` object for dynamic updates.
    color : State or str, optional
        The color of the component ("default", "inherit", "primary"). Default is "default".
        Can be a `State` object for dynamic updates.
    component : State or str, optional
        The component used for the root node (e.g., "QFrame"). Default is None (uses QFrame).
        Can be a `State` object for dynamic updates.
    disableGutters : State or bool, optional
        If True, removes gutters (padding). Default is False.
        Can be a `State` object for dynamic updates.
    disableSticky : State or bool, optional
        If True, the subheader will not stick to the top during scroll. Default is False.
        Can be a `State` object for dynamic updates.
    id : State or str, optional
        The identifier for the component. Default is None.
        Can be a `State` object for dynamic updates.
    inset : State or bool, optional
        If True, the subheader is indented. Default is False.
        Can be a `State` object for dynamic updates.
    sx : State, list, dict, Callable, str, or None, optional
        System prop for CSS overrides and additional styles. Default is None.
        Can be a `State` object for dynamic updates.
    size : State or str, optional
        The size of the subheader ("small", "medium", "large"). Default is "medium".
        Can be a `State` object for dynamic updates.
    **kwargs
        Additional keyword arguments passed to the parent `QFrame` class,
        supporting props of the native component (e.g., parent, style, className).

    Attributes
    ----------
    VALID_COLORS : list[str]
        Valid values for `color`: ["default", "inherit", "primary"].
    VALID_SIZES : list[str]
        Valid values for `size`: ["small", "medium", "large"].

    Notes
    -----
    - Props of the native component are supported via `**kwargs` (e.g., `parent`, `style`, `className`).
    - The `disableSticky` prop may require additional logic in Qt to emulate sticky behavior during scroll.
    - The `color` prop maps to theme palette: "default" uses `text.disabled`, "primary" uses `primary.main`, "inherit" retains parent color.

    Demos:
    - ListSubheader: https://qtmui.com/material-ui/qtmui-listsubheader/

    API Reference:
    - ListSubheader API: https://qtmui.com/material-ui/api/list-subheader/
    """

    VALID_COLORS = ["default", "inherit", "primary"]
    VALID_SIZES = ["small", "medium", "large"]

    def __init__(
        self,
        children: Optional[Union[State, str, QWidget, List[Union[str, QWidget]]]] = None,
        classes: Optional[Union[State, Dict]] = None,
        color: Union[State, str] = "default",
        component: Optional[Union[State, str]] = None,
        disableGutters: Union[State, bool] = False,
        disableSticky: Union[State, bool] = False,
        id: Optional[Union[State, str]] = None,
        inset: Union[State, bool] = False,
        sx: Optional[Union[State, List, Dict, Callable, str]] = None,
        size: Union[State, str] = "medium",
        **kwargs
    ):
        super().__init__()
        self.setObjectName(f"ListSubheader-{str(uuid.uuid4())}")

        self.theme = useTheme()
        self._widget_references = []

        # Set properties with validation
        self._set_children(children)
        self._set_classes(classes)
        self._set_color(color)
        self._set_component(component)
        self._set_disableGutters(disableGutters)
        self._set_disableSticky(disableSticky)
        self._set_id(id)
        self._set_inset(inset)
        self._set_sx(sx)
        self._set_size(size)

        self._init_ui()

    # Setter and Getter methods
    @_validate_param(file_path="qtmui.material.listsubheader", param_name="children", supported_signatures=Union[State, str, QWidget, List, type(None)])
    def _set_children(self, value):
        """Assign value to children and store references."""
        self._children = value

    def _get_children(self):
        """Get the children value."""
        return self._children.value if isinstance(self._children, State) else self._children

    @_validate_param(file_path="qtmui.material.listsubheader", param_name="classes", supported_signatures=Union[State, Dict, type(None)])
    def _set_classes(self, value):
        """Assign value to classes."""
        self._classes = value

    def _get_classes(self):
        """Get the classes value."""
        return self._classes.value if isinstance(self._classes, State) else self._classes

    @_validate_param(file_path="qtmui.material.listsubheader", param_name="color", supported_signatures=Union[State, str], valid_values=VALID_COLORS)
    def _set_color(self, value):
        """Assign value to color."""
        self._color = value

    def _get_color(self):
        """Get the color value."""
        return self._color.value if isinstance(self._color, State) else self._color

    @_validate_param(file_path="qtmui.material.listsubheader", param_name="component", supported_signatures=Union[State, str, type(None)])
    def _set_component(self, value):
        """Assign value to component."""
        self._component = value

    def _get_component(self):
        """Get the component value."""
        return self._component.value if isinstance(self._component, State) else self._component

    @_validate_param(file_path="qtmui.material.listsubheader", param_name="disableGutters", supported_signatures=Union[State, bool])
    def _set_disableGutters(self, value):
        """Assign value to disableGutters."""
        self._disableGutters = value

    def _get_disableGutters(self):
        """Get the disableGutters value."""
        return self._disableGutters.value if isinstance(self._disableGutters, State) else self._disableGutters

    @_validate_param(file_path="qtmui.material.listsubheader", param_name="disableSticky", supported_signatures=Union[State, bool])
    def _set_disableSticky(self, value):
        """Assign value to disableSticky."""
        self._disableSticky = value

    def _get_disableSticky(self):
        """Get the disableSticky value."""
        return self._disableSticky.value if isinstance(self._disableSticky, State) else self._disableSticky

    @_validate_param(file_path="qtmui.material.listsubheader", param_name="id", supported_signatures=Union[State, str, type(None)])
    def _set_id(self, value):
        """Assign value to id."""
        self._id = value

    def _get_id(self):
        """Get the id value."""
        return self._id.value if isinstance(self._id, State) else self._id

    @_validate_param(file_path="qtmui.material.listsubheader", param_name="inset", supported_signatures=Union[State, bool])
    def _set_inset(self, value):
        """Assign value to inset."""
        self._inset = value

    def _get_inset(self):
        """Get the inset value."""
        return self._inset.value if isinstance(self._inset, State) else self._inset

    @_validate_param(file_path="qtmui.material.listsubheader", param_name="sx", supported_signatures=Union[State, List, Dict, Callable, str, type(None)])
    def _set_sx(self, value):
        """Assign value to sx."""
        self._sx = value

    def _get_sx(self):
        """Get the sx value."""
        return self._sx.value if isinstance(self._sx, State) else self._sx

    @_validate_param(file_path="qtmui.material.listsubheader", param_name="size", supported_signatures=Union[State, str], valid_values=VALID_SIZES)
    def _set_size(self, value):
        """Assign value to size."""
        self._size = value

    def _get_size(self):
        """Get the size value."""
        return self._size.value if isinstance(self._size, State) else self._size

    def _init_ui(self):
        # Thêm nội dung children (nếu có)
        if self._children:
            # Tạo layout cho ListSubHeader
            self._layout = QVBoxLayout(self)
            self.setLayout(self._layout)
            self._layout.setContentsMargins(0, 0, 0, 0 if self._disableGutters else 10)
            if self._component:
                label = self._component(self)
            else:
                label = QLabel(self)
                
            label.setText(self._children)
            self._layout.addWidget(label)

        # Nếu inset=True, thêm padding bên trái
        if self._inset:
            self._layout.setContentsMargins(20, 0, 0, 0)

        padding = {
            "small": "4px 8px", # top-bottom left-right
            "medium": "8px 16px",
            "large": "12px 24px"
        }[self._size]

        self.setStyleSheet(self.styleSheet() + f"""
            QLabel {{
                font-size: 12px;
                padding: {padding}px;
                margin-left: 9px;
                color: {useTheme().palette.text.disabled};
            }}
        """)



      


import threading
import time
from typing import Optional, Union, Dict, List, Callable, Any
from PySide6.QtWidgets import QFrame, QVBoxLayout, QWidget
from PySide6.QtCore import Signal, QTimer
from qtmui.material.styles import useTheme
from qtmui.material.styles.create_theme.components.get_qss_styles import get_qss_style
from ..system.color_manipulator import alpha
from qtmui.hooks import State
from ..textfield import TextField
from ..utils.validate_params import _validate_param

class Autocomplete(QFrame):
    """
    A component that provides autocompletion functionality with a dropdown list of options.

    The `Autocomplete` component allows users to select one or multiple options from a list,
    with support for free input, custom rendering, and advanced filtering. It is built on top
    of `QWidget` and supports all props of the Material-UI `Autocomplete` component, including
    native component props via `**kwargs`.

    Parameters
    ----------
    options : State or list, optional
        A list of options that will be shown in the Autocomplete. Default is [].
        Can be a `State` object for dynamic updates.
    renderInput : State or Callable, optional
        Render the input component. Signature: function(params: dict) => QWidget.
        Default is None. Can be a `State` object for dynamic updates.
    autoComplete : State or bool, optional
        If True, the completion string appears inline after the input cursor.
        Default is False. Can be a `State` object for dynamic updates.
    autoHighlight : State or bool, optional
        If True, the first option is automatically highlighted. Default is False.
        Can be a `State` object for dynamic updates.
    autoSelect : State or bool, optional
        If True, the selected option becomes the input value on blur. Default is False.
        Can be a `State` object for dynamic updates.
    blurOnSelect : State, str, or bool, optional
        Controls if the input is blurred on selection. Valid values: "mouse", "touch",
        True, False. Default is False. Can be a `State` object for dynamic updates.
    ChipProps : State or dict, optional
        Props applied to the Chip element. Deprecated: Use `slotProps.chip` instead.
        Default is None. Can be a `State` object for dynamic updates.
    classes : State or dict, optional
        Override or extend the styles applied to the component. Default is None.
        Can be a `State` object for dynamic updates.
    clearIcon : State or QWidget, optional
        The icon to display for clearing the input. Default is None.
        Can be a `State` object for dynamic updates.
    clearOnBlur : State or bool, optional
        If True, the input's text is cleared on blur if no value is selected.
        Default is False. Can be a `State` object for dynamic updates.
    clearOnEscape : State or bool, optional
        If True, clears all values when the user presses Escape. Default is False.
        Can be a `State` object for dynamic updates.
    clearText : State or str, optional
        Text for the clear icon button. Default is "Clear".
        Can be a `State` object for dynamic updates.
    closeText : State or str, optional
        Text for the close popup icon button. Default is "Close".
        Can be a `State` object for dynamic updates.
    componentsProps : State or dict, optional
        Props for internal slots. Deprecated: Use `slotProps` instead. Default is None.
        Can be a `State` object for dynamic updates.
    defaultValue : State or Any, optional
        The default value. Default is None (or [] for multiple). Can be a `State` object.
    disableClearable : State or bool, optional
        If True, the input cannot be cleared. Default is False.
        Can be a `State` object for dynamic updates.
    disableCloseOnSelect : State or bool, optional
        If True, the popup stays open after selection. Default is False.
        Can be a `State` object for dynamic updates.
    disabled : State or bool, optional
        If True, the component is disabled. Default is False.
        Can be a `State` object for dynamic updates.
    disabledItemsFocusable : State or bool, optional
        If True, disabled items can be focused. Default is False.
        Can be a `State` object for dynamic updates.
    disableListWrap : State or bool, optional
        If True, the list box does not wrap focus. Default is False.
        Can be a `State` object for dynamic updates.
    disablePortal : State or bool, optional
        If True, the popup is rendered under the parent DOM hierarchy. Default is False.
        Can be a `State` object for dynamic updates.
    filterOptions : State or Callable, optional
        Function to filter options. Signature: function(options: List, state: dict) => List.
        Default is None. Can be a `State` object for dynamic updates.
    filterSelectedOptions : State or bool, optional
        If True, hides selected options from the list. Default is False.
        Can be a `State` object for dynamic updates.
    forcePopupIcon : State, str, or bool, optional
        Controls popup icon visibility. Valid values: "auto", True, False.
        Default is "auto". Can be a `State` object for dynamic updates.
    freeSolo : State or bool, optional
        If True, allows free input not bound to options. Default is False.
        Can be a `State` object for dynamic updates.
    fullWidth : State or bool, optional
        If True, the input takes the full width of its container. Default is False.
        Can be a `State` object for dynamic updates.
    getLimitTagsText : State or Callable, optional
        Function to display truncated tags label. Signature: function(more: int) => str.
        Default is None. Can be a `State` object for dynamic updates.
    getOptionDisabled : State or Callable, optional
        Function to determine if an option is disabled. Signature: function(option: Any) => bool.
        Default is None. Can be a `State` object for dynamic updates.
    getOptionKey : State or Callable, optional
        Function to get the key for an option. Signature: function(option: Any) => str | int.
        Default is None. Can be a `State` object for dynamic updates.
    getOptionLabel : State or Callable, optional
        Function to get the label for an option. Signature: function(option: Any) => str.
        Default is None. Can be a `State` object for dynamic updates.
    groupBy : State or Callable, optional
        Function to group options. Signature: function(option: Any) => str.
        Default is None. Can be a `State` object for dynamic updates.
    handleHomeEndKeys : State or bool, optional
        If True, handles Home/End keys when popup is open. Default is True.
        Can be a `State` object for dynamic updates.
    id : State or str, optional
        ID for the component. Default is None.
        Can be a `State` object for dynamic updates.
    includeInputInList : State or bool, optional
        If True, the input can be highlighted in the list. Default is False.
        Can be a `State` object for dynamic updates.
    inputValue : State or str, optional
        The input value. Default is None.
        Can be a `State` object for dynamic updates.
    isOptionEqualToValue : State or Callable, optional
        Function to compare option with value. Signature: function(option: Any, value: Any) => bool.
        Default is None. Can be a `State` object for dynamic updates.
    label : State or str, optional
        Label for the input. Default is None.
        Can be a `State` object for dynamic updates.
    limitTags : State or int, optional
        Maximum number of visible tags. Default is -1 (no limit).
        Can be a `State` object for dynamic updates.
    ListboxComponent : State or Any, optional
        Component for the listbox. Deprecated: Use `slots.listbox` instead.
        Default is None. Can be a `State` object for dynamic updates.
    ListboxProps : State or dict, optional
        Props for the Listbox. Deprecated: Use `slotProps.listbox` instead.
        Default is None. Can be a `State` object for dynamic updates.
    loading : State or bool, optional
        If True, shows a loading state. Default is False.
        Can be a `State` object for dynamic updates.
    loadingText : State or str, optional
        Text to display when loading. Default is "Loading...".
        Can be a `State` object for dynamic updates.
    multiple : State or bool, optional
        If True, allows multiple selections. Default is False.
        Can be a `State` object for dynamic updates.
    name : State or str, optional
        Name attribute for the input. Default is None.
        Can be a `State` object for dynamic updates.
    noOptionsText : State or str, optional
        Text to display when no options are available. Default is "No options".
        Can be a `State` object for dynamic updates.
    open : State or bool, optional
        If True, the popup is shown. Default is False.
        Can be a `State` object for dynamic updates.
    onChange : State or Callable, optional
        Callback fired when the value changes. Signature: function(event: Any, value: Any, reason: str, details: str) => None.
        Default is None. Can be a `State` object for dynamic updates.
    onClose : State or Callable, optional
        Callback fired when the popup closes. Signature: function(event: Any, reason: str) => None.
        Default is None. Can be a `State` object for dynamic updates.
    onHighlightChange : State or Callable, optional
        Callback fired when the highlighted option changes. Signature: function(event: Any, option: Any, reason: str) => None.
        Default is None. Can be a `State` object for dynamic updates.
    onInputChange : State or Callable, optional
        Callback fired when the input value changes. Signature: function(event: Any, value: str, reason: str) => None.
        Default is None. Can be a `State` object for dynamic updates.
    onOpen : State or Callable, optional
        Callback fired when the popup opens. Signature: function(event: Any) => None.
        Default is None. Can be a `State` object for dynamic updates.
    openOnFocus : State or bool, optional
        If True, the popup opens on input focus. Default is False.
        Can be a `State` object for dynamic updates.
    openText : State or str, optional
        Text for the open popup icon button. Default is "Open".
        Can be a `State` object for dynamic updates.
    PaperComponent : State or Any, optional
        Component for the popup body. Deprecated: Use `slots.paper` instead.
        Default is None. Can be a `State` object for dynamic updates.
    PopperComponent : State or Any, optional
        Component for positioning the popup. Deprecated: Use `slots.popper` instead.
        Default is None. Can be a `State` object for dynamic updates.
    popupIcon : State or QWidget, optional
        Icon for the popup. Default is None.
        Can be a `State` object for dynamic updates.
    readOnly : State or bool, optional
        If True, the component is read-only. Default is False.
        Can be a `State` object for dynamic updates.
    renderGroup : State or Callable, optional
        Function to render group headings. Signature: function(params: dict) => QWidget.
        Default is None. Can be a `State` object for dynamic updates.
    renderOption : State or Callable, optional
        Function to render each option. Signature: function(props: dict, option: Any, state: dict, ownerState: dict) => QWidget.
        Default is None. Can be a `State` object for dynamic updates.
    renderTags : State or Callable, optional
        Function to render selected tags. Deprecated: Use `renderValue` instead.
        Signature: function(value: List, getTagProps: Callable, ownerState: dict) => QWidget.
        Default is None. Can be a `State` object for dynamic updates.
    renderValue : State or Callable, optional
        Function to render selected value(s). Signature: function(value: Any, getItemProps: Callable, ownerState: dict) => QWidget.
        Default is None. Can be a `State` object for dynamic updates.
    selectOnFocus : State or bool, optional
        If True, selects the input text on focus. Default is True.
        Can be a `State` object for dynamic updates.
    selected : State or bool, optional
        If True, the input is visually selected. Default is False.
        Can be a `State` object for dynamic updates.
    size : State or str, optional
        Size of the component. Valid values: "small", "medium". Default is "medium".
        Can be a `State` object for dynamic updates.
    slotProps : State or dict, optional
        Props for each slot (e.g., chip, clearIndicator, listbox, paper, popper, popupIndicator).
        Default is None. Can be a `State` object for dynamic updates.
    slots : State or dict, optional
        Components for each slot (e.g., listbox, paper, popper). Default is None.
        Can be a `State` object for dynamic updates.
    sx : State, dict, Callable, str, or None, optional
        System prop for CSS overrides and additional styles. Default is None.
        Can be a `State` object for dynamic updates.
    value : State or Any, optional
        The value of the Autocomplete. Default is None.
        Can be a `State` object for dynamic updates.
    treeView : State or bool, optional
        If True, enables tree view mode. Default is False.
        Can be a `State` object for dynamic updates.
    **kwargs
        Additional keyword arguments passed to the parent `QWidget` class, supporting
        native component props (e.g., style, className).

    Attributes
    ----------
    VALID_SIZES : list[str]
        Valid values for the `size` parameter: ["small", "medium"].
    VALID_FORCE_POPUP_ICONS : list[str | bool]
        Valid values for the `forcePopupIcon` parameter: ["auto", True, False].
    VALID_BLUR_ON_SELECT : list[str | bool]
        Valid values for the `blurOnSelect` parameter: ["mouse", "touch", True, False].

    Signals
    -------
    onOpen : Signal
        Emitted when the popup opens.
    onClose : Signal
        Emitted when the popup closes.
    onChange : Signal
        Emitted when the value changes.
    onInputChange : Signal
        Emitted when the input value changes.
    onHighlightChange : Signal
        Emitted when the highlighted option changes.
    setupUi : Signal
        Emitted when the UI setup is complete.

    Notes
    -----
    - Props of the native component are supported via `**kwargs` (e.g., `style`, `className`).
    - Deprecated props (`ChipProps`, `componentsProps`, `ListboxComponent`, `ListboxProps`,
      `PaperComponent`, `PopperComponent`, `renderTags`) should be replaced with
      `slotProps.<slot>` and `slots.<slot>`.

    Demos:
    - Autocomplete: https://qtmui.com/material-ui/qtmui-autocomplete/

    API Reference:
    - Autocomplete API: https://qtmui.com/material-ui/api/autocomplete/
    """

    VALID_SIZES = ["small", "medium"]
    VALID_FORCE_POPUP_ICONS = ["auto", True, False]
    VALID_BLUR_ON_SELECT = ["mouse", "touch", True, False]

    def __init__(
        self,
        key: Optional[str] = None,
        options: Optional[Union[State, List]] = None,
        renderInput: Optional[Union[State, Callable]] = None,
        autoComplete: Union[State, bool] = False,
        autoHighlight: Union[State, bool] = False,
        autoSelect: Union[State, bool] = False,
        blurOnSelect: Union[State, str, bool] = False,
        ChipProps: Optional[Union[State, Dict]] = None,
        classes: Optional[Union[State, Dict]] = None,
        clearIcon: Optional[Union[State, QWidget]] = None,
        clearOnBlur: Union[State, bool] = False,
        clearOnEscape: Union[State, bool] = False,
        clearText: Union[State, str] = "Clear",
        closeText: Union[State, str] = "Close",
        componentsProps: Optional[Union[State, Dict]] = None,
        defaultValue: Optional[Union[State, Any]] = None,
        disableClearable: Union[State, bool] = False,
        disableCloseOnSelect: Union[State, bool] = False,
        disabled: Union[State, bool] = False,
        disabledItemsFocusable: Union[State, bool] = False,
        disableListWrap: Union[State, bool] = False,
        disablePortal: Union[State, bool] = False,
        filterOptions: Optional[Union[State, Callable]] = None,
        filterSelectedOptions: Union[State, bool] = False,
        forcePopupIcon: Union[State, str, bool] = "auto",
        freeSolo: Union[State, bool] = False,
        fullWidth: Union[State, bool] = False,
        getLimitTagsText: Optional[Union[State, Callable]] = None,
        getOptionDisabled: Optional[Union[State, Callable]] = None,
        getOptionKey: Optional[Union[State, Callable]] = None,
        getOptionLabel: Optional[Union[State, Callable[[Any], str]]] = None,
        groupBy: Optional[Union[State, Callable[[Any], str]]] = None,
        handleHomeEndKeys: Union[State, bool] = True,
        id: Optional[Union[State, str]] = None,
        includeInputInList: Union[State, bool] = False,
        inputValue: Optional[Union[State, str]] = None,
        isOptionEqualToValue: Optional[Union[State, Callable]] = None,
        label: Optional[Union[str, State, Callable]] = None,
        limitTags: Union[State, int] = -1,
        ListboxComponent: Optional[Union[State, Any]] = None,
        ListboxProps: Optional[Union[State, Dict]] = None,
        loading: Union[State, bool] = False,
        loadingText: Union[State, str] = "Loading...",
        multiple: Union[State, bool] = False,
        name: Optional[Union[State, str]] = None,
        noOptionsText: Union[State, str] = "No options",
        open: Union[State, bool] = False,
        onChange: Optional[Union[State, Callable]] = None,
        onClose: Optional[Union[State, Callable]] = None,
        onHighlightChange: Optional[Union[State, Callable]] = None,
        onInputChange: Optional[Union[State, Callable]] = None,
        onOpen: Optional[Union[State, Callable[[Any], None]]] = None,
        openOnFocus: Union[State, bool] = False,
        openText: Union[State, str] = "Open",
        PaperComponent: Optional[Union[State, Any]] = None,
        PopperComponent: Optional[Union[State, Any]] = None,
        popupIcon: Optional[Union[State, QWidget]] = None,
        readOnly: Union[State, bool] = False,
        renderGroup: Optional[Union[State, Callable]] = None,
        renderOption: Optional[Union[State, Callable]] = None,
        renderTags: Optional[Union[State, Callable]] = None,
        renderValue: Optional[Union[State, Callable]] = None,
        selectOnFocus: Union[State, bool] = True,
        selected: Optional[Union[State, bool]] = False,
        size: Union[State, str] = "medium",
        slotProps: Optional[Union[State, Dict]] = None,
        slots: Optional[Union[State, Dict]] = None,
        sx: Optional[Union[State, Dict, Callable, str]] = None,
        value: Optional[Union[State, Any]] = None,
        treeView: Union[State, bool] = False,
        **kwargs
    ):
        super().__init__()
        self._key = key
        self._name = name

        # Thiết lập các thuộc tính với dấu gạch dưới
        self._set_options(options or [])
        self._set_renderInput(renderInput)
        self._set_autoComplete(autoComplete)
        self._set_autoHighlight(autoHighlight)
        self._set_autoSelect(autoSelect)
        self._set_blurOnSelect(blurOnSelect)
        self._set_ChipProps(ChipProps)
        self._set_classes(classes)
        self._set_clearIcon(clearIcon)
        self._set_clearOnBlur(clearOnBlur)
        self._set_clearOnEscape(clearOnEscape)
        self._set_clearText(clearText)
        self._set_closeText(closeText)
        self._set_componentsProps(componentsProps)
        self._set_defaultValue(defaultValue)
        self._set_disableClearable(disableClearable)
        self._set_disableCloseOnSelect(disableCloseOnSelect)
        self._set_disabled(disabled)
        self._set_disabledItemsFocusable(disabledItemsFocusable)
        self._set_disableListWrap(disableListWrap)
        self._set_disablePortal(disablePortal)
        self._set_filterOptions(filterOptions)
        self._set_filterSelectedOptions(filterSelectedOptions)
        self._set_forcePopupIcon(forcePopupIcon)
        self._set_freeSolo(freeSolo)
        self._set_fullWidth(fullWidth)
        self._set_getLimitTagsText(getLimitTagsText)
        self._set_getOptionDisabled(getOptionDisabled)
        self._set_getOptionKey(getOptionKey)
        self._set_getOptionLabel(getOptionLabel)
        self._set_groupBy(groupBy)
        self._set_handleHomeEndKeys(handleHomeEndKeys)
        self._set_id(id)
        self._set_includeInputInList(includeInputInList)
        self._set_inputValue(inputValue)
        self._set_isOptionEqualToValue(isOptionEqualToValue)
        self._set_label(label)
        self._set_limitTags(limitTags)
        self._set_ListboxComponent(ListboxComponent)
        self._set_ListboxProps(ListboxProps)
        self._set_loading(loading)
        self._set_loadingText(loadingText)
        self._set_multiple(multiple)
        self._set_name(name)
        self._set_noOptionsText(noOptionsText)
        self._set_open(open)
        self._set_onChange(onChange)
        self._set_onClose(onClose)
        self._set_onHighlightChange(onHighlightChange)
        self._set_onInputChange(onInputChange)
        self._set_onOpen(onOpen)
        self._set_openOnFocus(openOnFocus)
        self._set_openText(openText)
        self._set_PaperComponent(PaperComponent)
        self._set_PopperComponent(PopperComponent)
        self._set_popupIcon(popupIcon)
        self._set_readOnly(readOnly)
        self._set_renderGroup(renderGroup)
        self._set_renderOption(renderOption)
        self._set_renderTags(renderTags)
        self._set_renderValue(renderValue)
        self._set_selectOnFocus(selectOnFocus)
        self._validate_selected(selected)
        self._set_size(size)
        self._set_slotProps(slotProps)
        self._set_slots(slots)
        self._set_sx(sx)
        self._set_value(value)
        self._set_treeView(treeView)

        self._state = self._get_value()
        if self._get_fullWidth():
            if not self._get_sx():
                self._set_sx({})
            self._get_sx().update({"width": "100%"})

        self._init_ui()


    # Setter và Getter cho tất cả các tham số
    @_validate_param(file_path="qtmui.material.autocomplete", param_name="options", supported_signatures=Union[State, List, type(None)])
    def _set_options(self, value):
        self._options = value

    def _get_options(self):
        return self._options.value if isinstance(self._options, State) else self._options

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="renderInput", supported_signatures=Union[State, Callable, type(None)])
    def _set_renderInput(self, value):
        self._renderInput = value

    def _get_renderInput(self):
        return self._renderInput.value if isinstance(self._renderInput, State) else self._renderInput

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="autoComplete", supported_signatures=Union[State, bool])
    def _set_autoComplete(self, value):
        self._autoComplete = value

    def _get_autoComplete(self):
        return self._autoComplete.value if isinstance(self._autoComplete, State) else self._autoComplete

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="autoHighlight", supported_signatures=Union[State, bool])
    def _set_autoHighlight(self, value):
        self._autoHighlight = value

    def _get_autoHighlight(self):
        return self._autoHighlight.value if isinstance(self._autoHighlight, State) else self._autoHighlight

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="autoSelect", supported_signatures=Union[State, bool])
    def _set_autoSelect(self, value):
        self._autoSelect = value

    def _get_autoSelect(self):
        return self._autoSelect.value if isinstance(self._autoSelect, State) else self._autoSelect

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="blurOnSelect", supported_signatures=Union[State, str, bool], valid_values=VALID_BLUR_ON_SELECT)
    def _set_blurOnSelect(self, value):
        self._blurOnSelect = value

    def _get_blurOnSelect(self):
        return self._blurOnSelect.value if isinstance(self._blurOnSelect, State) else self._blurOnSelect

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="ChipProps", supported_signatures=Union[State, Dict, type(None)])
    def _set_ChipProps(self, value):
        self._ChipProps = value

    def _get_ChipProps(self):
        return self._ChipProps.value if isinstance(self._ChipProps, State) else self._ChipProps

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="classes", supported_signatures=Union[State, Dict, type(None)])
    def _set_classes(self, value):
        self._classes = value

    def _get_classes(self):
        return self._classes.value if isinstance(self._classes, State) else self._classes

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="clearIcon", supported_signatures=Union[State, QWidget, type(None)])
    def _set_clearIcon(self, value):
        self._clearIcon = value

    def _get_clearIcon(self):
        return self._clearIcon.value if isinstance(self._clearIcon, State) else self._clearIcon

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="clearOnBlur", supported_signatures=Union[State, bool])
    def _set_clearOnBlur(self, value):
        self._clearOnBlur = value

    def _get_clearOnBlur(self):
        return self._clearOnBlur.value if isinstance(self._clearOnBlur, State) else self._clearOnBlur

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="clearOnEscape", supported_signatures=Union[State, bool])
    def _set_clearOnEscape(self, value):
        self._clearOnEscape = value

    def _get_clearOnEscape(self):
        return self._clearOnEscape.value if isinstance(self._clearOnEscape, State) else self._clearOnEscape

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="clearText", supported_signatures=Union[State, str])
    def _set_clearText(self, value):
        self._clearText = value

    def _get_clearText(self):
        return self._clearText.value if isinstance(self._clearText, State) else self._clearText

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="closeText", supported_signatures=Union[State, str])
    def _set_closeText(self, value):
        self._closeText = value

    def _get_closeText(self):
        return self._closeText.value if isinstance(self._closeText, State) else self._closeText

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="componentsProps", supported_signatures=Union[State, Dict, type(None)])
    def _set_componentsProps(self, value):
        self._componentsProps = value

    def _get_componentsProps(self):
        return self._componentsProps.value if isinstance(self._componentsProps, State) else self._componentsProps

    # @_validate_param(file_path="qtmui.material.autocomplete", param_name="defaultValue", supported_signatures=Union[State, Any, type(None)])
    def _set_defaultValue(self, value):
        self._defaultValue = value

    def _get_defaultValue(self):
        return self._defaultValue.value if isinstance(self._defaultValue, State) else self._defaultValue

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="disableClearable", supported_signatures=Union[State, bool])
    def _set_disableClearable(self, value):
        self._disableClearable = value

    def _get_disableClearable(self):
        return self._disableClearable.value if isinstance(self._disableClearable, State) else self._disableClearable

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="disableCloseOnSelect", supported_signatures=Union[State, bool])
    def _set_disableCloseOnSelect(self, value):
        self._disableCloseOnSelect = value

    def _get_disableCloseOnSelect(self):
        return self._disableCloseOnSelect.value if isinstance(self._disableCloseOnSelect, State) else self._disableCloseOnSelect

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="disabled", supported_signatures=Union[State, bool])
    def _set_disabled(self, value):
        self._disabled = value

    def _get_disabled(self):
        return self._disabled.value if isinstance(self._disabled, State) else self._disabled

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="disabledItemsFocusable", supported_signatures=Union[State, bool])
    def _set_disabledItemsFocusable(self, value):
        self._disabledItemsFocusable = value

    def _get_disabledItemsFocusable(self):
        return self._disabledItemsFocusable.value if isinstance(self._disabledItemsFocusable, State) else self._disabledItemsFocusable

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="disableListWrap", supported_signatures=Union[State, bool])
    def _set_disableListWrap(self, value):
        self._disableListWrap = value

    def _get_disableListWrap(self):
        return self._disableListWrap.value if isinstance(self._disableListWrap, State) else self._disableListWrap

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="disablePortal", supported_signatures=Union[State, bool])
    def _set_disablePortal(self, value):
        self._disablePortal = value

    def _get_disablePortal(self):
        return self._disablePortal.value if isinstance(self._disablePortal, State) else self._disablePortal

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="filterOptions", supported_signatures=Union[State, Callable, type(None)])
    def _set_filterOptions(self, value):
        self._filterOptions = value

    def _get_filterOptions(self):
        return self._filterOptions.value if isinstance(self._filterOptions, State) else self._filterOptions

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="filterSelectedOptions", supported_signatures=Union[State, bool])
    def _set_filterSelectedOptions(self, value):
        self._filterSelectedOptions = value

    def _get_filterSelectedOptions(self):
        return self._filterSelectedOptions.value if isinstance(self._filterSelectedOptions, State) else self._filterSelectedOptions

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="forcePopupIcon", supported_signatures=Union[State, str, bool], valid_values=VALID_FORCE_POPUP_ICONS)
    def _set_forcePopupIcon(self, value):
        self._forcePopupIcon = value

    def _get_forcePopupIcon(self):
        return self._forcePopupIcon.value if isinstance(self._forcePopupIcon, State) else self._forcePopupIcon

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="freeSolo", supported_signatures=Union[State, bool])
    def _set_freeSolo(self, value):
        self._freeSolo = value

    def _get_freeSolo(self):
        return self._freeSolo.value if isinstance(self._freeSolo, State) else self._freeSolo

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="fullWidth", supported_signatures=Union[State, bool])
    def _set_fullWidth(self, value):
        self._fullWidth = value

    def _get_fullWidth(self):
        return self._fullWidth.value if isinstance(self._fullWidth, State) else self._fullWidth

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="getLimitTagsText", supported_signatures=Union[State, Callable, type(None)])
    def _set_getLimitTagsText(self, value):
        self._getLimitTagsText = value

    def _get_getLimitTagsText(self):
        return self._getLimitTagsText.value if isinstance(self._getLimitTagsText, State) else self._getLimitTagsText

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="getOptionDisabled", supported_signatures=Union[State, Callable, type(None)])
    def _set_getOptionDisabled(self, value):
        self._getOptionDisabled = value

    def _get_getOptionDisabled(self):
        return self._getOptionDisabled.value if isinstance(self._getOptionDisabled, State) else self._getOptionDisabled

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="getOptionKey", supported_signatures=Union[State, Callable, type(None)])
    def _set_getOptionKey(self, value):
        self._getOptionKey = value

    def _get_getOptionKey(self):
        return self._getOptionKey.value if isinstance(self._getOptionKey, State) else self._getOptionKey

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="getOptionLabel", supported_signatures=Union[State, Callable, type(None)])
    def _set_getOptionLabel(self, value):
        self._getOptionLabel = value

    def _get_getOptionLabel(self):
        return self._getOptionLabel.value if isinstance(self._getOptionLabel, State) else self._getOptionLabel

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="groupBy", supported_signatures=Union[State, Callable, type(None)])
    def _set_groupBy(self, value):
        self._groupBy = value

    def _get_groupBy(self):
        return self._groupBy.value if isinstance(self._groupBy, State) else self._groupBy

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="handleHomeEndKeys", supported_signatures=Union[State, bool])
    def _set_handleHomeEndKeys(self, value):
        self._handleHomeEndKeys = value

    def _get_handleHomeEndKeys(self):
        return self._handleHomeEndKeys.value if isinstance(self._handleHomeEndKeys, State) else self._handleHomeEndKeys

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="id", supported_signatures=Union[State, str, type(None)])
    def _set_id(self, value):
        self._id = value

    def _get_id(self):
        return self._id.value if isinstance(self._id, State) else self._id

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="includeInputInList", supported_signatures=Union[State, bool])
    def _set_includeInputInList(self, value):
        self._includeInputInList = value

    def _get_includeInputInList(self):
        return self._includeInputInList.value if isinstance(self._includeInputInList, State) else self._includeInputInList

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="inputValue", supported_signatures=Union[State, str, type(None)])
    def _set_inputValue(self, value):
        self._inputValue = value

    def _get_inputValue(self):
        return self._inputValue.value if isinstance(self._inputValue, State) else self._inputValue

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="isOptionEqualToValue", supported_signatures=Union[State, Callable, type(None)])
    def _set_isOptionEqualToValue(self, value):
        self._isOptionEqualToValue = value

    def _get_isOptionEqualToValue(self):
        return self._isOptionEqualToValue.value if isinstance(self._isOptionEqualToValue, State) else self._isOptionEqualToValue

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="label", supported_signatures=Union[State, str, type(None)])
    def _set_label(self, value):
        self._label = value

    def _get_label(self):
        return self._label.value if isinstance(self._label, State) else self._label

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="limitTags", supported_signatures=Union[State, int])
    def _set_limitTags(self, value):
        self._limitTags = value

    def _get_limitTags(self):
        return self._limitTags.value if isinstance(self._limitTags, State) else self._limitTags

    # @_validate_param(file_path="qtmui.material.autocomplete", param_name="ListboxComponent", supported_signatures=Union[State, Any, type(None)])
    def _set_ListboxComponent(self, value):
        self._ListboxComponent = value

    def _get_ListboxComponent(self):
        return self._ListboxComponent.value if isinstance(self._ListboxComponent, State) else self._ListboxComponent

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="ListboxProps", supported_signatures=Union[State, Dict, type(None)])
    def _set_ListboxProps(self, value):
        self._ListboxProps = value

    def _get_ListboxProps(self):
        return self._ListboxProps.value if isinstance(self._ListboxProps, State) else self._ListboxProps

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="loading", supported_signatures=Union[State, bool])
    def _set_loading(self, value):
        self._loading = value

    def _get_loading(self):
        return self._loading.value if isinstance(self._loading, State) else self._loading

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="loadingText", supported_signatures=Union[State, str])
    def _set_loadingText(self, value):
        self._loadingText = value

    def _get_loadingText(self):
        return self._loadingText.value if isinstance(self._loadingText, State) else self._loadingText

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="multiple", supported_signatures=Union[State, bool])
    def _set_multiple(self, value):
        self._multiple = value

    def _get_multiple(self):
        return self._multiple.value if isinstance(self._multiple, State) else self._multiple

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="name", supported_signatures=Union[State, str, type(None)])
    def _set_name(self, value):
        self._name = value

    def _get_name(self):
        return self._name.value if isinstance(self._name, State) else self._name

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="noOptionsText", supported_signatures=Union[State, str])
    def _set_noOptionsText(self, value):
        self._noOptionsText = value

    def _get_noOptionsText(self):
        return self._noOptionsText.value if isinstance(self._noOptionsText, State) else self._noOptionsText

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="open", supported_signatures=Union[State, bool])
    def _set_open(self, value):
        self._open = value

    def _get_open(self):
        return self._open.value if isinstance(self._open, State) else self._open

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="onChange", supported_signatures=Union[State, Callable, type(None)])
    def _set_onChange(self, value):
        self._onChange = value

    def _get_onChange(self):
        return self._onChange.value if isinstance(self._onChange, State) else self._onChange

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="onClose", supported_signatures=Union[State, Callable, type(None)])
    def _set_onClose(self, value):
        self._onClose = value

    def _get_onClose(self):
        return self._onClose.value if isinstance(self._onClose, State) else self._onClose

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="onHighlightChange", supported_signatures=Union[State, Callable, type(None)])
    def _set_onHighlightChange(self, value):
        self._onHighlightChange = value

    def _get_onHighlightChange(self):
        return self._onHighlightChange.value if isinstance(self._onHighlightChange, State) else self._onHighlightChange

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="onInputChange", supported_signatures=Union[State, Callable, type(None)])
    def _set_onInputChange(self, value):
        self._onInputChange = value

    def _get_onInputChange(self):
        return self._onInputChange.value if isinstance(self._onInputChange, State) else self._onInputChange

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="onOpen", supported_signatures=Union[State, Callable, type(None)])
    def _set_onOpen(self, value):
        self._onOpen = value

    def _get_onOpen(self):
        return self._onOpen.value if isinstance(self._onOpen, State) else self._onOpen

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="openOnFocus", supported_signatures=Union[State, bool])
    def _set_openOnFocus(self, value):
        self._openOnFocus = value

    def _get_openOnFocus(self):
        return self._openOnFocus.value if isinstance(self._openOnFocus, State) else self._openOnFocus

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="openText", supported_signatures=Union[State, str])
    def _set_openText(self, value):
        self._openText = value

    def _get_openText(self):
        return self._openText.value if isinstance(self._openText, State) else self._openText

    # @_validate_param(file_path="qtmui.material.autocomplete", param_name="PaperComponent", supported_signatures=Union[State, Any, type(None)])
    def _set_PaperComponent(self, value):
        self._PaperComponent = value

    def _get_PaperComponent(self):
        return self._PaperComponent.value if isinstance(self._PaperComponent, State) else self._PaperComponent

    # @_validate_param(file_path="qtmui.material.autocomplete", param_name="PopperComponent", supported_signatures=Union[State, Any, type(None)])
    def _set_PopperComponent(self, value):
        self._PopperComponent = value

    def _get_PopperComponent(self):
        return self._PopperComponent.value if isinstance(self._PopperComponent, State) else self._PopperComponent

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="popupIcon", supported_signatures=Union[State, QWidget, type(None)])
    def _set_popupIcon(self, value):
        self._popupIcon = value

    def _get_popupIcon(self):
        return self._popupIcon.value if isinstance(self._popupIcon, State) else self._popupIcon

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="readOnly", supported_signatures=Union[State, bool])
    def _set_readOnly(self, value):
        self._readOnly = value

    def _get_readOnly(self):
        return self._readOnly.value if isinstance(self._readOnly, State) else self._readOnly

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="renderGroup", supported_signatures=Union[State, Callable, type(None)])
    def _set_renderGroup(self, value):
        self._renderGroup = value

    def _get_renderGroup(self):
        return self._renderGroup.value if isinstance(self._renderGroup, State) else self._renderGroup

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="renderOption", supported_signatures=Union[State, Callable, type(None)])
    def _set_renderOption(self, value):
        self._renderOption = value

    def _get_renderOption(self):
        return self._renderOption.value if isinstance(self._renderOption, State) else self._renderOption

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="renderTags", supported_signatures=Union[State, Callable, type(None)])
    def _set_renderTags(self, value):
        self._renderTags = value

    def _get_renderTags(self):
        return self._renderTags.value if isinstance(self._renderTags, State) else self._renderTags

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="renderValue", supported_signatures=Union[State, Callable, type(None)])
    def _set_renderValue(self, value):
        self._renderValue = value

    def _get_renderValue(self):
        return self._renderValue.value if isinstance(self._renderValue, State) else self._renderValue

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="selectOnFocus", supported_signatures=Union[State, bool])
    def _set_selectOnFocus(self, value):
        self._selectOnFocus = value

    def _get_selectOnFocus(self):
        return self._selectOnFocus.value if isinstance(self._selectOnFocus, State) else self._selectOnFocus

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="selected", supported_signatures=Union[State, bool, type(None)])
    def _validate_selected(self, value):
        self._selected = value

    def _get_selected(self):
        return self._selected.value if isinstance(self._selected, State) else self._selected

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="size", supported_signatures=Union[State, str], valid_values=VALID_SIZES)
    def _set_size(self, value):
        self._size = value

    def _get_size(self):
        return self._size.value if isinstance(self._size, State) else self._size

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="slotProps", supported_signatures=Union[State, Dict, type(None)])
    def _set_slotProps(self, value):
        self._slotProps = value

    def _get_slotProps(self):
        return self._slotProps.value if isinstance(self._slotProps, State) else self._slotProps

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="slots", supported_signatures=Union[State, Dict, type(None)])
    def _set_slots(self, value):
        self._slots = value

    def _get_slots(self):
        return self._slots.value if isinstance(self._slots, State) else self._slots

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="sx", supported_signatures=Union[State, Dict, Callable, str, type(None)])
    def _set_sx(self, value):
        self._sx = value

    def _get_sx(self):
        return self._sx.value if isinstance(self._sx, State) else self._sx

    # @_validate_param(file_path="qtmui.material.autocomplete", param_name="value", supported_signatures=Union[State, Any, type(None)])
    def _set_value(self, value):
        self._value = value

    def _get_value(self):
        return self._value.value if isinstance(self._value, State) else self._value

    @_validate_param(file_path="qtmui.material.autocomplete", param_name="treeView", supported_signatures=Union[State, bool])
    def _set_treeView(self, value):
        self._treeView = value

    def _get_treeView(self):
        return self._treeView.value if isinstance(self._treeView, State) else self._treeView

    def _init_ui(self):
        self.setLayout(QVBoxLayout())
        self.layout().setContentsMargins(0,0,0,0)

        input_params = {
            # "label": self._label,
            "key": self._key,
            "select": True,
            "autoComplete": True,
            "disabled": self._disabled,
            "defaultValue": self._defaultValue,
            "fullWidth": self._fullWidth,
            "id": self._id,
            "inputValue": self._inputValue,
            "options": self._options,
            "onChange": self._onChange,
            "onInputChange": self._onInputChange,
            "treeView": self._treeView,
            "multiple": self._multiple,
            "getOptionLabel": self._getOptionLabel,
            "renderOption": self._renderOption,
            "renderTags": self._renderTags,
            "size": self._size,
            "value": self._value,
            # **kwargs
        }
        self._inputField = self._renderInput(input_params)
        self.layout().addWidget(self._inputField)

        self.valueChanged = self._inputField.valueChanged

 
